@@ -48,7 +48,7 @@ $string['aggregateonlygraded_help'] = 'An empty grade is a grade which is missin
 This setting determines whether empty grades are not included in the aggregation or are counted as minimal grades, for example 0 for an assignment graded between 0 and 100.';
 $string['aggregateoutcomes'] = 'Include outcomes in aggregation';
 $string['aggregateoutcomes_help'] = 'If enabled, outcomes are included in the aggregation. This may result in an unexpected category total.';
-$string['aggregatesonly'] = 'Change to aggregates only';
+$string['aggregatesonly'] = 'Show totals only';
 $string['aggregatesum'] = 'Natural';
 $string['aggregateweightedmean'] = 'Weighted mean of grades';
 $string['aggregateweightedmean2'] = 'Simple weighted mean of grades';
@@ -111,6 +111,7 @@ $string['calculationadd'] = 'Add calculation';
 $string['calculationedit'] = 'Edit calculation';
 $string['calculationsaved'] = 'Calculation saved';
 $string['calculationview'] = 'View calculation';
+$string['calculationwarning'] = 'This category uses a grade calculation for the category total rather than an aggregation.';
 $string['cannotaccessgroup'] = 'Can not access grades of selected group, sorry.';
 $string['categories'] = 'Categories';
 $string['category'] = 'Category';
@@ -119,12 +120,12 @@ $string['categoryname'] = 'Category name';
 $string['categorytotal'] = 'Category total';
 $string['categorytotalname'] = 'Category total name';
 $string['categorytotalfull'] = '{$a->category} total';
+$string['selectuserlink'] = 'Select a user';
 $string['collapsecriterion'] = 'Collapse criterion';
 $string['compact'] = 'Compact';
 $string['componentcontrolsvisibility'] = 'Whether this grade item is hidden is controlled by the activity settings.';
 $string['contract'] = 'Contract category';
 $string['contributiontocoursetotal'] = 'Contribution to course total';
-$string['controls'] = 'Controls';
 $string['courseavg'] = 'Course average';
 $string['coursegradecategory'] = 'Course grade category';
 $string['coursegradedisplaytype'] = 'Course grade display type';
@@ -175,27 +176,28 @@ $string['editoutcomes'] = 'Edit outcomes';
 $string['editscale'] = 'Edit scale';
 $string['edittree'] = 'Setup';
 $string['editverbose'] = 'Edit {$a->category} {$a->itemmodule} {$a->itemname}';
-$string['enableajax'] = 'Enable AJAX';
-$string['enableajax_help'] = 'Adds a layer of AJAX functionality to the grader report, simplifying and speeding up common operations. Depends on Javascript being switched on at the user\'s browser level.';
 $string['enableoutcomes'] = 'Enable outcomes';
 $string['enableoutcomes_help'] = 'If enabled, grade items may be graded using one or more scales tied to outcome statements.';
 $string['encoding'] = 'Encoding';
 $string['encoding_help'] = 'Select the character encoding used for the data. (The standard encoding is UTF-8.) If the wrong encoding is selected by mistake, it will be noticeable when previewing the data for import.';
 $string['errorcalculationnoequal'] = 'Formula must start with equal sign (=1+2)';
 $string['errorcalculationunknown'] = 'Invalid formula';
-$string['errorcalculationbroken'] = 'Probably circular reference or broken calculation formula';
+$string['errorcalculationbroken'] = 'Error in the calculation of grade item {$a}.';
 $string['errorgradevaluenonnumeric'] = 'Received non-numeric for low or high grade for';
 $string['errornocalculationallowed'] = 'Calculations are not allowed for this item';
 $string['errornocategorisedid'] = 'Could not get an uncategorised id!';
 $string['errornocourse'] = 'Could not get course information';
+$string['erroroverweight'] = 'Weight total exceeds 100%.';
 $string['errorreprintheadersnonnumeric'] = 'Received non-numeric value for reprint-headers';
 $string['errorsavegrade'] = 'Could not save grade, sorry.';
 $string['errorsettinggrade'] = 'Error saving "{$a->itemname}" grade for userid {$a->userid}';
+$string['errorunderweight'] = 'Weight total is less than 100%.';
 $string['errorupdatinggradecategoryaggregateonlygraded'] = 'Error updating the "Aggregate only non-empty grades" setting of grade category ID {$a->id}';
 $string['errorupdatinggradecategoryaggregateoutcomes'] = 'Error updating the "Include outcomes in aggregation" setting of grade category ID {$a->id}';
 $string['errorupdatinggradecategoryaggregation'] = 'Error updating the aggregation type of grade category ID {$a->id}';
 $string['errorupdatinggradeitemaggregationcoef'] = 'Error updating the aggregation coefficient (weight or extra credit) of grade item ID {$a->id}';
 $string['eventgradedeleted'] = 'Grade deleted';
+$string['eventgradeexported'] = 'Grade exported';
 $string['eventgradeitemcreated'] = 'Grade item created';
 $string['eventgradeitemdeleted'] = 'Grade item deleted';
 $string['eventgradeitemupdated'] = 'Grade item updated';
@@ -212,6 +214,7 @@ $string['excluded_help'] = 'If ticked, the grade will not be included in any agg
 $string['expand'] = 'Expand category';
 $string['expandcriterion'] = 'Expand criterion';
 $string['export'] = 'Export';
+$string['exportas'] = 'Export as';
 $string['exportalloutcomes'] = 'Export all outcomes';
 $string['exportfeedback'] = 'Include feedback in export';
 $string['exportfeedback_desc'] = 'This can be overridden during export.';
@@ -233,10 +236,15 @@ $string['feedbackedit'] = 'Edit feedback';
 $string['feedbackfiles'] = 'Feedback files';
 $string['feedbackforgradeitems'] = 'Feedback for {$a}';
 $string['feedbackhistoryfiles'] = 'Feedback history files';
+$string['feedbackprovided'] = 'Feedback provided';
 $string['feedbacks'] = 'Feedbacks';
 $string['feedbacksaved'] = 'Feedback saved';
 $string['feedbackview'] = 'View feedback';
 $string['finalgrade'] = 'Final grade';
+$string['filterbothactive'] = 'First ({$a->first}) Last ({$a->last})';
+$string['filterbyname'] = 'Filter by name';
+$string['filterfirstactive'] = 'First ({$a->first})';
+$string['filterlastactive'] = 'Last ({$a->last})';
 $string['finalgrade_help'] = 'If the overridden checkbox is ticked, a grade may be added or amended.';
 $string['fixedstudents'] = 'Static students column';
 $string['fixedstudents_help'] = 'Allows grades to scroll horizontally without losing sight of the students column, by making it static.';
@@ -247,7 +255,7 @@ $string['forceon'] = 'Force: On';
 $string['forelementtypes'] = 'for the selected {$a}';
 $string['forstudents'] = 'For students';
 $string['full'] = 'Full';
-$string['fullmode'] = 'Change to full view';
+$string['fullmode'] = 'Show grades and totals';
 $string['generalsettings'] = 'General settings';
 $string['grade'] = 'Grade';
 $string['grade_help'] = 'The grade to award the student for their work.';
@@ -261,6 +269,7 @@ $string['gradebookcalculationswarning'] = 'Note: Some errors have been detected
 
 The newer version is {$a->currentversion}; you are using gradebook version {$a->gradebookversion}. You can see a list of changes in <a href="{$a->url}">Gradebook calculation changes</a>.';
 $string['gradebookhiddenerror'] = 'The gradebook is currently set to hide everything from students.';
+$string['gradebooknavigationmenu'] = 'Gradebook navigation menu';
 $string['gradebookhistories'] = 'Grade histories';
 $string['gradebooksetup'] = 'Gradebook setup';
 $string['gradeboundary'] = 'Letter grade boundary';
@@ -343,8 +352,7 @@ $string['gradereport'] = 'Grade report';
 $string['graderreport'] = 'Grader report';
 $string['grades'] = 'Grades';
 $string['gradesforuser'] = 'Grades for {$a->user}';
-$string['singleview'] = 'Single view for {$a}';
-$string['gradesonly'] = 'Change to grades only';
+$string['gradesonly'] = 'Show grades only';
 $string['gradesmoduledeletionpendingwarning'] = 'Warning: Activity deletion in progress! Some grades are about to be removed.';
 $string['gradesmoduledeletionprefix'] = '[Deletion in progress]';
 $string['gradessettings'] = 'Grade settings';
@@ -358,6 +366,7 @@ $string['gradetype_help'] = 'There are 4 grade types:
 
 Only value and scale grade types may be aggregated. The grade type for an activity-based grade item is set on the activity settings page.';
 $string['gradevaluetoobig'] = 'One of the grade values is larger than the allowed grade maximum of {$a}';
+$string['gradevaluetoosmall'] = 'One of the grade values is smaller than the allowed grade minimum of {$a}';
 $string['gradeview'] = 'View grade';
 $string['gradewasmodifiedduringediting'] = 'The grade entered for {$a->itemname} for {$a->username} was ignored because it was more recently updated by someone else.';
 $string['gradeweighthelp'] = 'Grade weight help';
@@ -400,6 +409,7 @@ $string['identifier'] = 'Identify user by';
 $string['idnumbers'] = 'ID numbers';
 $string['ignore'] = 'Ignore';
 $string['import'] = 'Import';
+$string['importas'] = 'Import as';
 $string['importcsv'] = 'Import CSV';
 $string['importcsv_help'] = 'Grades can be imported via a CSV file with format as follows:
 
@@ -501,7 +511,7 @@ $string['minmaxupgradefixbutton'] = 'Resolve inconsistencies';
 $string['minmaxupgradewarning'] = 'Note: An inconsistency has been detected with some grades due to a change in the minimum and maximum grades used when calculating the grade displayed in the gradebook. It is recommended that the inconsistency is resolved by clicking the button below, though this will result in some grades being changed.';
 $string['minimum_show'] = 'Show minimum grade';
 $string['minimum_show_help'] = 'Minimum grade is used in calculating grades and weights. If not shown, minimum grade will default to zero and cannot be edited.';
-$string['missingitemtypeoreid'] = 'Array key (itemtype or eid) missing from 2nd param of grade_edit_tree_column_select::get_item_cell($item, $params)';
+$string['missingitemtypeoreid'] = 'Array key (itemtype or eid) missing';
 $string['missingscale'] = 'Scale must be selected';
 $string['mode'] = 'Mode';
 $string['modgrade'] = 'Grade';
@@ -534,7 +544,8 @@ $string['modgradetypenone'] = 'None';
 $string['modgradetypepoint'] = 'Point';
 $string['modgradetypescale'] = 'Scale';
 $string['morethanmax'] = 'The grade entered for {$a->itemname} for {$a->username} is more than the maximum allowed';
-$string['moveselectedto'] = 'Move selected items to';
+$string['movesitems'] = 'Move items';
+$string['moveselectedto'] = 'Move selected items to:';
 $string['movingelement'] = 'Moving {$a}';
 $string['multfactor'] = 'Multiplicator';
 $string['multfactorvalue'] = 'Multiplicator value for {$a}';
@@ -543,6 +554,7 @@ $string['mygrades'] = 'User menu grades link';
 $string['mygrades_desc'] = 'This setting allows for the option of linking to an external gradebook from the user menu.';
 $string['mypreferences'] = 'My preferences';
 $string['myreportpreferences'] = 'My report preferences';
+$string['myself'] = 'Myself';
 $string['mustchooserescaleyesorno'] = 'You must choose whether to rescale existing grades or not.';
 $string['neverdeletehistory'] = 'Never delete history';
 $string['newcategory'] = 'New category';
@@ -562,7 +574,8 @@ $string['noidnumber'] = 'No ID number';
 $string['nolettergrade'] = 'No letter grade for';
 $string['nomode'] = 'NA';
 $string['nonnumericweight'] = 'Received non-numeric value for';
-$string['nonunlockableverbose'] = 'This grade cannot be unlocked until {$a->itemname} is unlocked.';
+$string['nonunlockableverbose'] = 'To unlock this grade, you need to unlock {$a->itemname}.';
+$string['nonunhideableverbose'] = 'To unhide this grade, you need to unhide {$a->itemname}.';
 $string['nonweightedpct'] = 'non-weighted %';
 $string['nooutcome'] = 'No outcome';
 $string['nooutcomes'] = 'Outcome items must be linked to a course outcome, but there are no outcomes for this course. Would you like to add one?';
@@ -582,6 +595,7 @@ $string['onascaleof'] = 'on a scale of {$a->grademin} to {$a->grademax}';
 $string['operations'] = 'Operations';
 $string['options'] = 'Options';
 $string['others'] = 'Others';
+$string['otheruser'] = 'User';
 $string['outcome'] = 'Outcome';
 $string['outcome_help'] = 'This setting determines the outcome which this grade item will represent in the gradebook.';
 $string['outcomeassigntocourse'] = 'Assign another outcome to this course';
@@ -600,6 +614,7 @@ $string['outcomescoursecustom'] = 'Custom used (no remove)';
 $string['outcomescoursenotused'] = 'Standard not used';
 $string['outcomescourseused'] = 'Standard used (no remove)';
 $string['outcomescustom'] = 'Custom outcomes';
+$string['outcomesdisabled'] = 'Outcomes are not enabled';
 $string['outcomeshortname'] = 'Short name';
 $string['outcomesstandard'] = 'Standard outcomes';
 $string['outcomesstandardavailable'] = 'Available standard outcomes';
@@ -721,11 +736,13 @@ $string['savechanges'] = 'Save changes';
 $string['savepreferences'] = 'Save preferences';
 $string['scaleconfirmdelete'] = 'Are you sure you wish to delete the scale "{$a}"?';
 $string['scaledpct'] = 'Scaled %';
+$string['searchcollapsedcolumns'] = 'Search collapsed columns';
 $string['seeallcoursegrades'] = 'See all course grades';
 $string['select'] = 'Select {$a}';
 $string['selectalloroneuser'] = 'Select all or one user';
 $string['selectauser'] = 'Select a user';
 $string['selectdestination'] = 'Select destination of {$a}';
+$string['selectgrouplink'] = 'Select a group';
 $string['separator'] = 'Separator';
 $string['separator_help'] = 'Select the separator used in the CSV file. (Normally this is a comma.)';
 $string['sepcolon'] = 'Colon';
@@ -740,9 +757,6 @@ $string['setting'] = 'Setting';
 $string['settings'] = 'Settings';
 $string['setup'] = 'Setup';
 $string['setweights'] = 'Set weights';
-$string['showanalysisicon'] = 'Show grade analysis icon';
-$string['showanalysisicon_desc'] = 'Whether to show grade analysis icon by default. If the activity module supports it, the grade analysis icon links to a page with more detailed explanation of the grade and how it was obtained.';
-$string['showanalysisicon_help'] = 'If the activity module supports it, the grade analysis icon links to a page with more detailed explanation of the grade and how it was obtained.';
 $string['showaverage'] = 'Show average';
 $string['showaverage_help'] = 'Whether to show an average column. Note that participants may be able to estimate others\' grades if the average is calculated from a small number of grades. For performance reasons the average is approximate if it is dependent on any hidden items.';
 $string['showcontributiontocoursetotal'] = 'Show contribution to course total';
@@ -751,8 +765,8 @@ $string['showfeedback'] = 'Show feedback';
 $string['showfeedback_help'] = 'Whether to show a column for feedback.';
 $string['showgrade'] = 'Show grades';
 $string['showgrade_help'] = 'Whether to show a column for grades.';
-$string['showingaggregatesonly'] = 'Showing aggregates only';
-$string['showingfullmode'] = 'Showing full view';
+$string['showingaggregatesonly'] = 'Showing totals only';
+$string['showingfullmode'] = 'Showing grades and totals';
 $string['showinggradesonly'] = 'Showing grades only';
 $string['showlettergrade'] = 'Show letter grades';
 $string['showlettergrade_help'] = 'Whether to show a column for letter grades.';
@@ -762,16 +776,11 @@ $string['showweight'] = 'Show weightings';
 $string['showweight_help'] = 'Whether to show a column for the grade weight.';
 $string['rangedecimals'] = 'Range decimal places';
 $string['rangedecimals_help'] = 'The number of decimal places to display for the range.';
-$string['showactivityicons'] = 'Show activity icons';
-$string['showactivityicons_help'] = 'If enabled, activity icons are shown next to activity names.';
+$string['showallgrades'] = 'Show all grades';
 $string['showallhidden'] = 'Show hidden';
 $string['showallstudents'] = 'Show all Students';
 $string['showaverages'] = 'Show column averages';
 $string['showaverages_help'] = 'If enabled, the grader report will contain an additional row displaying the average (mean) for each category and grade item.';
-$string['showcalculations'] = 'Show calculations';
-$string['showcalculations_help'] = 'If enabled, when editing, a calculator icon is shown for each grade item and category with a visual indicator that a grade item is calculated.';
-$string['showeyecons'] = 'Show show/hide icons';
-$string['showeyecons_help'] = 'If enabled, when editing is turned on, a show/hide icon is shown for each grade for controlling its visibility to the student.';
 $string['showgroups'] = 'Show groups';
 $string['showhiddenitems'] = 'Show hidden items';
 $string['showhiddenitems_help'] = 'Whether hidden grade items are hidden entirely or if the names of hidden grade items are visible to students.
@@ -780,8 +789,6 @@ $string['showhiddenitems_help'] = 'Whether hidden grade items are hidden entirel
 * Only hidden until - Grade items with a "hide until" date set are hidden completely until the set date, after which the whole item is shown
 * Do not show - Hidden grade items are completely hidden';
 $string['showhiddenuntilonly'] = 'Only hidden until';
-$string['showlocks'] = 'Show locks';
-$string['showlocks_help'] = 'If enabled, when editing is turned on, a lock/unlock icon is shown for each grade for controlling whether the grade can be automatically updated by the related activity.';
 $string['shownohidden'] = 'Do not show';
 $string['shownooutcomes'] = 'Hide outcomes';
 $string['shownumberofgrades'] = 'Show number of grades in averages';
@@ -790,10 +797,6 @@ $string['showonlyactiveenrol'] = 'Show only active enrolments';
 $string['showonlyactiveenrol_help'] = 'This setting determines, if only active enrolled users will be visible in gradebook report. If enabled suspended users will not be shown in gradebook.';
 $string['showpercentage'] = 'Show percentage';
 $string['showpercentage_help'] = 'Whether to show the percentage value of each grade item.';
-$string['showquickfeedback'] = 'Show quick feedback';
-$string['showquickfeedback_help'] = 'If enabled, when editing is turned on, a feedback text input box with a dotted border appears for each grade, allowing the feedback for many grades to be edited at the same time. Changes are saved and highlighted when the update button is clicked.
-
-Note that when feedback is edited in the grader report, an overridden flag is set, meaning that the feedback can no longer be changed from within the related activity.';
 $string['showranges'] = 'Show ranges';
 $string['showranges_help'] = 'If enabled, the grader report will contain an additional row displaying the range for each category and grade item.';
 $string['showrank'] = 'Show rank';
@@ -812,8 +815,6 @@ $string['standarddeviation'] = 'Standard deviation';
 $string['stats'] = 'Statistics';
 $string['statslink'] = 'Stats';
 $string['student'] = 'Student';
-$string['studentsperpage'] = 'Students per page';
-$string['studentsperpage_help'] = 'This setting determines the number of students displayed per page in the grader report.';
 $string['studentsperpagereduced'] = 'Reduced maximum students per page from {$a->originalstudentsperpage} to {$a->studentsperpage}. Consider increasing the PHP setting max_input_vars from {$a->maxinputvars}.';
 $string['subcategory'] = 'Normal category';
 $string['submissions'] = 'Submissions';
@@ -866,7 +867,9 @@ $string['userenrolmentsuspended'] = 'User enrolment suspended';
 $string['useweighted'] = 'Use weighted';
 $string['verbosescales'] = 'Verbose scales';
 $string['verbosescales_help'] = 'A verbose scale uses words rather than numbers. Set to \'Yes\' if both numerical and verbose scales are to be imported. Set to \'No\' if only numerical scales are to be imported.';
+$string['viewas'] = 'View report as';
 $string['viewbygroup'] = 'Group';
+$string['viewfeedback'] = 'View feedback';
 $string['viewgrades'] = 'View grades';
 $string['weight'] = 'weight';
 $string['weight_help'] = 'A value used to determine the relative value of multiple grade items in a category or course.';
@@ -889,8 +892,33 @@ $string['xml'] = 'XML';
 $string['yes'] = 'Yes';
 $string['yourgrade'] = 'Your grade';
 
+$string['aria-toggledropdown'] = 'Toggle the following dropdown';
+$string['aria:dropdowngrades'] = 'Grade items found';
+
 // Deprecated since Moodle 4.0.
 $string['navmethod'] = 'Navigation method';
 $string['dropdown'] = 'Drop-down menu';
 $string['tabs'] = 'Tabs';
 $string['combo'] = 'Tabs and drop-down menu';
+
+// Deprecated since Moodle 4.2.
+$string['showanalysisicon'] = 'Show grade analysis icon';
+$string['showanalysisicon_desc'] = 'Whether to show grade analysis icon by default. If the activity module supports it, the grade analysis icon links to a page with more detailed explanation of the grade and how it was obtained.';
+$string['showanalysisicon_help'] = 'If the activity module supports it, the grade analysis icon links to a page with more detailed explanation of the grade and how it was obtained.';
+$string['showlocks'] = 'Show locks';
+$string['showlocks_help'] = 'If enabled, when editing is turned on, a lock/unlock icon is shown for each grade for controlling whether the grade can be automatically updated by the related activity.';
+$string['showeyecons'] = 'Show show/hide icons';
+$string['showeyecons_help'] = 'If enabled, when editing is turned on, a show/hide icon is shown for each grade for controlling its visibility to the student.';
+$string['showquickfeedback'] = 'Show quick feedback';
+$string['showquickfeedback_help'] = 'If enabled, when editing is turned on, a feedback text input box with a dotted border appears for each grade, allowing the feedback for many grades to be edited at the same time. Changes are saved and highlighted when the update button is clicked.
+
+Note that when feedback is edited in the grader report, an overridden flag is set, meaning that the feedback can no longer be changed from within the related activity.';
+$string['enableajax'] = 'Enable AJAX';
+$string['enableajax_help'] = 'Adds a layer of AJAX functionality to the grader report, simplifying and speeding up common operations. Depends on Javascript being switched on at the user\'s browser level.';
+$string['controls'] = 'Controls';
+$string['showactivityicons'] = 'Show activity icons';
+$string['showactivityicons_help'] = 'If enabled, activity icons are shown next to activity names.';
+$string['showcalculations'] = 'Show calculations';
+$string['showcalculations_help'] = 'If enabled, when editing, a calculator icon is shown for each grade item and category with a visual indicator that a grade item is calculated.';
+$string['studentsperpage'] = 'Students per page';
+$string['studentsperpage_help'] = 'This setting determines the number of students displayed per page in the grader report.';
