@@ -1,69 +1,44 @@
 <?php
-/* 
-version V5.17 17 May 2012  (c) 2000-2012  John Lim (jlim#natsoft.com).  All rights
-reserved.
-  Released under both BSD license and Lesser GPL library license. 
-  Whenever there is any discrepancy between the two licenses, 
-  the BSD license will take precedence. 
-Set tabs to 4 for best viewing.
-
-  Latest version is available at http://adodb.sourceforge.net
-
-  21.02.2002 - Wade Johnson wade@wadejohnson.de
-			   Extended ODBC class for Sybase SQLAnywhere.
-   1) Added support to retrieve the last row insert ID on tables with
-	  primary key column using autoincrement function.
-
-   2) Added blob support.  Usage:
-		 a) create blob variable on db server:
-
-		$dbconn->create_blobvar($blobVarName);
-
-	  b) load blob var from file.  $filename must be complete path
-
-	  $dbcon->load_blobvar_from_file($blobVarName, $filename);
-
-	  c) Use the $blobVarName in SQL insert or update statement in the values
-	  clause:
-
-		$recordSet = $dbconn->Execute('INSERT INTO tabname (idcol, blobcol) '
-		.
-	   'VALUES (\'test\', ' . $blobVarName . ')');
-
-	 instead of loading blob from a file, you can also load from 
-	  an unformatted (raw) blob variable:
-	  $dbcon->load_blobvar_from_var($blobVarName, $varName);
-
-	  d) drop blob variable on db server to free up resources:
-	  $dbconn->drop_blobvar($blobVarName);
-
-  Sybase_SQLAnywhere data driver. Requires ODBC.
-
-*/
+/**
+ * SAP SQL Anywhere driver (previously Sybase SQL Anywhere)
+ *
+ * This file is part of ADOdb, a Database Abstraction Layer library for PHP.
+ *
+ * @package ADOdb
+ * @link https://adodb.org Project's web site and documentation
+ * @link https://github.com/ADOdb/ADOdb Source code and issue tracker
+ *
+ * The ADOdb Library is dual-licensed, released under both the BSD 3-Clause
+ * and the GNU Lesser General Public Licence (LGPL) v2.1 or, at your option,
+ * any later version. This means you can use it in proprietary products.
+ * See the LICENSE.md file distributed with this source code for details.
+ * @license BSD-3-Clause
+ * @license LGPL-2.1-or-later
+ *
+ * @copyright 2000-2013 John Lim
+ * @copyright 2014 Damien Regad, Mark Newnham and the ADOdb community
+ */
 
 // security - hide paths
 if (!defined('ADODB_DIR')) die();
 
 if (!defined('_ADODB_ODBC_LAYER')) {
- include(ADODB_DIR."/drivers/adodb-odbc.inc.php");
+ include_once(ADODB_DIR."/drivers/adodb-odbc.inc.php");
 }
 
 if (!defined('ADODB_SYBASE_SQLANYWHERE')){
 
  define('ADODB_SYBASE_SQLANYWHERE',1);
 
- class ADODB_sqlanywhere extends ADODB_odbc {
-  	var $databaseType = "sqlanywhere";	
-	var $hasInsertID = true;
-	
-	function ADODB_sqlanywhere()
-	{
-		$this->ADODB_odbc();
-	}
+ class ADODB_sqlanywhere extends ADODB_odbc
+ {
+	 var $databaseType = "sqlanywhere";
+	 var $hasInsertID = true;
 
-	 function _insertid() {
-  	   return $this->GetOne('select @@identity');
-	 }
+	protected function _insertID($table = '', $column = '')
+	{
+		 return $this->GetOne('select @@identity');
+	}
 
   function create_blobvar($blobVarName) {
    $this->Execute("create variable $blobVarName long binary");
@@ -152,18 +127,12 @@ if (!defined('ADODB_SYBASE_SQLANYWHERE')){
   }
  }; //class
 
- class  ADORecordSet_sqlanywhere extends ADORecordSet_odbc {	
+ class  ADORecordSet_sqlanywhere extends ADORecordSet_odbc {
 
-  var $databaseType = "sqlanywhere";		
-
- function ADORecordSet_sqlanywhere($id,$mode=false)
- {
-  $this->ADORecordSet_odbc($id,$mode);
- }
+  var $databaseType = "sqlanywhere";
 
 
  }; //class
 
 
 } //define
-?>
