@@ -116,12 +116,12 @@ Y.namespace('M.atto_recordrtc').Button = Y.Base.create('button', Y.M.editor_atto
                 focusAfterHide: null
             });
 
-            // If dialogue is closed during recording, do the following.
             dialogue.after('visibleChange', function() {
                 var closed = !dialogue.get('visible'),
                     m = M.atto_recordrtc.commonmodule;
 
                 if (closed) {
+                    // If dialogue is closed during recording, do the following.
                     window.clearInterval(m.countdownTicker);
 
                     if (m.mediaRecorder && m.mediaRecorder.state !== 'inactive') {
@@ -135,9 +135,14 @@ Y.namespace('M.atto_recordrtc').Button = Y.Base.create('button', Y.M.editor_atto
                             }
                         });
                     }
+
+                    // Because the player uses ids to identify things (this should be fixed)
+                    // we must make sure the dialogue contents only exist once in the DOM.
+                    // Therefore, when a dialogue is closed, we must remove its contents.
+                    this.getDialogue().set('bodyContent', '');
                 }
 
-            });
+            }, this);
 
             dialogue.on('click', function() {
                 this.centered();
@@ -177,7 +182,9 @@ Y.namespace('M.atto_recordrtc').Button = Y.Base.create('button', Y.M.editor_atto
      * @private
      */
     _audio: function() {
-        var dialogue = this.getDialogue();
+        var dialogue = this.getDialogue({
+            focusAfterHide: 'audio'
+        });
 
         dialogue.set('headerContent', M.util.get_string('audiortc', 'atto_recordrtc'));
         dialogue.set('bodyContent', this._createContent('audio'));
@@ -194,7 +201,9 @@ Y.namespace('M.atto_recordrtc').Button = Y.Base.create('button', Y.M.editor_atto
      * @private
      */
     _video: function() {
-        var dialogue = this.getDialogue();
+        var dialogue = this.getDialogue({
+            focusAfterHide: 'video'
+        });
 
         dialogue.set('headerContent', M.util.get_string('videortc', 'atto_recordrtc'));
         dialogue.set('bodyContent', this._createContent('video'));
@@ -242,7 +251,9 @@ Y.namespace('M.atto_recordrtc').Button = Y.Base.create('button', Y.M.editor_atto
      * @param {Object} scope The "this" context of the editor.
      */
     closeDialogue: function(scope) {
-        scope.getDialogue().hide();
+        scope.getDialogue().hide({
+            focusAfterHide: null
+        });
 
         scope.editor.focus();
     },
@@ -255,7 +266,9 @@ Y.namespace('M.atto_recordrtc').Button = Y.Base.create('button', Y.M.editor_atto
      * @param {string} annotation The HTML link to the recording.
      */
     setLink: function(scope, annotation) {
-        scope.getDialogue().hide();
+        scope.getDialogue().hide({
+            focusAfterHide: null
+        });
 
         scope.editor.focus();
         scope.get('host').insertContentAtFocusPoint(annotation);
@@ -314,12 +327,32 @@ Y.namespace('M.atto_recordrtc').Button = Y.Base.create('button', Y.M.editor_atto
         },
 
         /**
-         * The timelimit to use when generating this recordrtc.
+         * The audiotimelimit to use when generating this recordrtc.
          *
-         * @attribute timelimit
+         * @attribute audiotimelimit
          * @type String
          */
-        timelimit: {
+        audiotimelimit: {
+            value: null
+        },
+
+        /**
+         * The videotimelimit to use when generating this recordrtc.
+         *
+         * @attribute videotimelimit
+         * @type String
+         */
+        videotimelimit: {
+            value: null
+        },
+
+        /**
+         * The defaulttimelimit to use when generating this recordrtc.
+         *
+         * @attribute defaulttimelimit
+         * @type String
+         */
+        defaulttimelimit: {
             value: null
         },
 
