@@ -1,10 +1,3 @@
-/*
-YUI 3.13.0 (build 508226d)
-Copyright 2013 Yahoo! Inc. All rights reserved.
-Licensed under the BSD License.
-http://yuilibrary.com/license/
-*/
-
 YUI.add('autocomplete-plugin', function (Y, NAME) {
 
 /**
@@ -52,4 +45,4 @@ Plugin.AutoComplete     = ACListPlugin;
 Plugin.AutoCompleteList = ACListPlugin;
 
 
-}, '3.13.0', {"requires": ["autocomplete-list", "node-pluginhost"]});
+}, '3.18.1', {"requires": ["autocomplete-list", "node-pluginhost"]});
