@@ -40,6 +40,15 @@ class media_videojs_plugin extends core_media_player_native {
     protected $extensions = null;
     /** @var bool is this a youtube link */
     protected $youtube = false;
+    /** @var bool Need to use Ogv.JS Tech plugin or not. */
+    protected $ogvtech = false;
+    /** @var array Ogv.JS supported extensions */
+    protected $ogvsupportedextensions = [
+        '.ogv',
+        '.webm',
+        '.oga',
+        '.ogg'
+    ];
 
     /**
      * Generates code required to embed the player.
@@ -72,10 +81,8 @@ class media_videojs_plugin extends core_media_player_native {
             $hasposter = self::get_attribute($text, 'poster') !== null;
         }
 
-        // Currently Flash in VideoJS does not support responsive layout. If Flash is enabled try to guess
-        // if HTML5 player will be engaged for the user and then set it to responsive.
-        $responsive = (get_config('media_videojs', 'useflash') && !$this->youtube) ? null : true;
-        $flashtech = false;
+        // Try to guess if HTML5 player will be engaged for the user and then set it to responsive.
+        $responsive = (!$this->youtube) ? null : true;
 
         // Build list of source tags.
         foreach ($urls as $url) {
@@ -101,10 +108,6 @@ class media_videojs_plugin extends core_media_player_native {
             if ($responsive === null) {
                 $responsive = core_useragent::supports_html5($extension);
             }
-            if (($url->get_scheme() === 'rtmp' || !core_useragent::supports_html5($extension))
-                    && get_config('media_videojs', 'useflash')) {
-                $flashtech = true;
-            }
         }
         $sources = implode("\n", $sources);
 
@@ -123,8 +126,10 @@ class media_videojs_plugin extends core_media_player_native {
 
             $sources = ''; // Do not specify <source> tags - it may confuse browser.
             $isaudio = false; // Just in case.
-        } else if ($flashtech) {
-            $datasetup[] = '"techOrder": ["flash", "html5"]';
+        }
+
+        if ($this->ogvtech) {
+            $datasetup[] = '"techOrder": ["OgvJS"]';
         }
 
         // Add a language.
@@ -149,6 +154,10 @@ class media_videojs_plugin extends core_media_player_native {
             $datasetup[] = '"aspectRatio": "1:0"';
         }
 
+        // Additional setup for playback rate and user actions.
+        $datasetup[] = '"playbackRates": [0.5, 0.75, 1, 1.25, 1.5, 1.75, 2]';
+        $datasetup[] = '"userActions": {"hotkeys": true}';
+
         // Attributes for the video/audio tag.
         // We use data-setup-lazy as the attribute name for the config instead of
         // data-setup because data-setup will cause video.js to load the player as soon as the library is loaded,
@@ -207,7 +216,7 @@ class media_videojs_plugin extends core_media_player_native {
             }
         }
 
-        return html_writer::div($text, 'mediaplugin mediaplugin_videojs');
+        return html_writer::div($text, 'mediaplugin mediaplugin_videojs d-block');
     }
 
     /**
@@ -252,15 +261,17 @@ class media_videojs_plugin extends core_media_player_native {
         global $CFG;
         require_once($CFG->libdir . '/filelib.php');
         if ($this->extensions === null) {
+            // Get extensions set by user in UI config.
             $filetypes = preg_split('/\s*,\s*/',
                 strtolower(trim(get_config('media_videojs', 'videoextensions') . ',' .
                 get_config('media_videojs', 'audioextensions'))));
+
             $this->extensions = file_get_typegroup('extension', $filetypes);
-            if ($this->extensions && !get_config('media_videojs', 'useflash')) {
-                // If Flash is disabled only return extensions natively supported by browsers.
-                $nativeextensions = array_merge(file_get_typegroup('extension', 'html_video'),
-                    file_get_typegroup('extension', 'html_audio'));
-                $this->extensions = array_intersect($this->extensions, $nativeextensions);
+            if ($this->extensions) {
+                // Get extensions supported by player.
+                $supportedextensions = array_merge(file_get_typegroup('extension', 'html_video'),
+                    file_get_typegroup('extension', 'html_audio'), file_get_typegroup('extension', 'media_source'));
+                $this->extensions = array_intersect($this->extensions, $supportedextensions);
             }
         }
         return $this->extensions;
@@ -274,35 +285,37 @@ class media_videojs_plugin extends core_media_player_native {
             $url = reset($urls);
 
             // Check against regex.
-            if (preg_match($this->get_regex_youtube(), $url->out(false), $this->matches)) {
+            if (preg_match($this->get_regex_youtube(), $url->out(false), $matches)) {
                 $this->youtube = true;
                 return array($url);
             }
         }
 
-        // If Flash fallback is enabled we can not check if/when browser supports flash.
         $extensions = $this->get_supported_extensions();
-        $rtmpallowed = get_config('media_videojs', 'rtmp') && get_config('media_videojs', 'useflash');
         foreach ($urls as $url) {
-            // If RTMP support is disabled, skip the URL that is using RTMP (which
-            // might have been picked to the list by its valid extension).
-            if (!$rtmpallowed && ($url->get_scheme() === 'rtmp')) {
+            // Skip the URL that is using RTMP (which might have been picked to the list by its valid extension).
+            if ($url->get_scheme() === 'rtmp') {
                 continue;
             }
-            // If RTMP support is allowed, URL with RTMP scheme is supported irrespective to extension.
-            if ($rtmpallowed && ($url->get_scheme() === 'rtmp')) {
+
+            $ext = '.' . core_media_manager::instance()->get_extension($url);
+            // Handle HLS and MPEG-DASH if supported.
+            $isstream = in_array($ext, file_get_typegroup('extension', 'media_source'));
+            if ($isstream && in_array($ext, $extensions) && core_useragent::supports_media_source_extensions($ext)) {
                 $result[] = $url;
                 continue;
             }
 
-            if (!get_config('media_videojs', 'useflash')) {
-                return parent::list_supported_urls($urls, $options);
-            } else {
-                $ext = core_media_manager::instance()->get_extension($url);
-                if (in_array('.' . $ext, $extensions)) {
-                    $result[] = $url;
-                }
+            // Ogv.JS Tech.
+            $this->ogvtech = false;
+            if (in_array($ext, $this->ogvsupportedextensions) &&
+                (core_useragent::is_safari() || core_useragent::is_ios())) {
+                $this->ogvtech = true;
+                $result[] = $url;
+                continue;
             }
+
+            return parent::list_supported_urls($urls, $options);
         }
         return $result;
     }
@@ -329,33 +342,40 @@ class media_videojs_plugin extends core_media_player_native {
         $langfiles = get_directory_list($basedir);
         $candidates = [];
         foreach ($langfiles as $langfile) {
-            if (strtolower(pathinfo($langfile, PATHINFO_EXTENSION)) !== 'js') {
+            if (strtolower(pathinfo($langfile, PATHINFO_EXTENSION)) !== 'json') {
                 continue;
             }
-            $lang = basename($langfile, '.js');
-            if (strtolower($langfile) === $this->language . '.js') {
-                // Found an exact match for the language.
-                $js = file_get_contents($basedir . $langfile);
-                break;
+            $lang = basename($langfile, '.json');
+            if (strtolower($langfile) === $this->language . '.json') {
+                // Found an exact match for the language. It is stored in $this->language.
+                return;
             }
             if (substr($this->language, 0, 2) === strtolower(substr($langfile, 0, 2))) {
                 // Not an exact match but similar, for example "pt_br" is similar to "pt".
                 $candidates[$lang] = $langfile;
             }
         }
-        if (empty($js) && $candidates) {
+
+        if ($candidates) {
             // Exact match was not found, take the first candidate.
             $this->language = key($candidates);
-            $js = file_get_contents($basedir . $candidates[$this->language]);
-        }
-        // Add it as a language for Video.JS.
-        if (!empty($js)) {
-            return "$js\n";
+        } else {
+            // Could not match, use default language of video player (English).
+            $this->language = 'en';
         }
+    }
 
-        // Could not match, use default language of video player (English).
-        $this->language = null;
-        return "";
+    /**
+     * Returns the requested language pack in the json format.
+     *
+     * @param string $lang The language code
+     * @return false|string The read data or false on failure
+     */
+    public static function get_language_content(string $lang) {
+        global $CFG;
+        $langfile = "{$CFG->dirroot}/media/player/videojs/videojs/lang/{$lang}.json";
+
+        return file_exists($langfile) ? file_get_contents($langfile) : '';
     }
 
     public function supports($usedextensions = []) {
@@ -363,9 +383,6 @@ class media_videojs_plugin extends core_media_player_native {
         if (get_config('media_videojs', 'youtube')) {
             $supports .= ($supports ? '<br>' : '') . get_string('youtube', 'media_videojs');
         }
-        if (get_config('media_videojs', 'rtmp') && get_config('media_videojs', 'useflash')) {
-            $supports .= ($supports ? '<br>' : '') . get_string('rtmp', 'media_videojs');
-        }
         return $supports;
     }
 
@@ -375,10 +392,6 @@ class media_videojs_plugin extends core_media_player_native {
         if (get_config('media_videojs', 'youtube')) {
             $markers = array_merge($markers, array('youtube.com', 'youtube-nocookie.com', 'youtu.be', 'y2u.be'));
         }
-        // Add RTMP support if enabled.
-        if (get_config('media_videojs', 'rtmp') && get_config('media_videojs', 'useflash')) {
-            $markers[] = 'rtmp://';
-        }
 
         return $markers;
     }
@@ -406,18 +419,17 @@ class media_videojs_plugin extends core_media_player_native {
      * @param moodle_page $page The page we are going to add requirements to.
      */
     public function setup($page) {
+        if (during_initial_install() || is_major_upgrade_required()) {
+            return;
+        }
 
         // Load dynamic loader. It will scan page for videojs media and load necessary modules.
         // Loader will be loaded on absolutely every page, however the videojs will only be loaded
         // when video is present on the page or added later to it in AJAX.
-        $path = new moodle_url('/media/player/videojs/videojs/video-js.swf');
-        $contents = 'videojs.options.flash.swf = "' . $path . '";' . "\n";
-        $contents .= $this->find_language(current_language());
+        $this->find_language();
         $page->requires->js_amd_inline(<<<EOT
 require(["media_videojs/loader"], function(loader) {
-    loader.setUp(function(videojs) {
-        $contents
-    });
+    loader.setUp('$this->language');
 });
 EOT
         );
