@@ -72,7 +72,7 @@ class submission_locked extends base {
      */
     public function get_description() {
         return "The user with id '$this->userid' locked the submission for the user with id '$this->relateduserid' for " .
-            "the assignment with the course module id '$this->contextinstanceid'.";
+            "the assignment with course module id '$this->contextinstanceid'.";
     }
 
     /**
@@ -95,18 +95,6 @@ class submission_locked extends base {
         $this->data['objecttable'] = 'assign';
     }
 
-    /**
-     * Return legacy data for add_to_log().
-     *
-     * @return array
-     */
-    protected function get_legacy_logdata() {
-        $user = $this->get_record_snapshot('user', $this->relateduserid);
-        $logmessage = get_string('locksubmissionforstudent', 'assign', array('id' => $user->id, 'fullname' => fullname($user)));
-        $this->set_legacy_logdata('lock submission', $logmessage);
-        return parent::get_legacy_logdata();
-    }
-
     /**
      * Custom validation.
      *
@@ -123,4 +111,8 @@ class submission_locked extends base {
             throw new \coding_exception('The \'relateduserid\' must be set.');
         }
     }
+
+    public static function get_objectid_mapping() {
+        return array('db' => 'assign', 'restore' => 'assign');
+    }
 }
