@@ -12,7 +12,6 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
-/* global SELECTOR, COMMENTCOLOUR, COMMENTTEXTCOLOUR */
 
 /**
  * Provides an in browser PDF editor.
@@ -160,7 +159,7 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
     this.draw = function(focus) {
         var drawable = new M.assignfeedback_editpdf.drawable(this.editor),
             node,
-            drawingregion = this.editor.get_dialogue_element(SELECTOR.DRAWINGREGION),
+            drawingcanvas = this.editor.get_dialogue_element(SELECTOR.DRAWINGCANVAS),
             container,
             label,
             marker,
@@ -203,7 +202,7 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
             color: COMMENTTEXTCOLOUR
         });
 
-        drawingregion.append(container);
+        drawingcanvas.append(container);
         container.setStyle('position', 'absolute');
         container.setX(position.x);
         container.setY(position.y);
@@ -215,11 +214,7 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
             'height': scrollheight + 'px',
             'overflow': 'hidden'
         });
-        marker.setStyles({
-            'position': 'absolute',
-            'bottom': 0 - scrollheight + 'px',
-            'color': COMMENTCOLOUR[this.colour]
-        });
+        marker.setStyle('color', COMMENTCOLOUR[this.colour]);
         this.attach_events(node, menu);
         if (focus) {
             node.focus();
@@ -237,11 +232,20 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
      * @method delete_comment_later
      */
     this.delete_comment_later = function() {
-        if (this.deleteme) {
+        if (this.deleteme && !this.is_menu_active()) {
             this.remove();
         }
     };
 
+    /**
+     * Returns true if the menu is active, false otherwise.
+     *
+     * @return bool true if menu is active, else false.
+     */
+    this.is_menu_active = function() {
+        return this.menu !== null && this.menu.get('visible');
+    };
+
     /**
      * Comment nodes have a bunch of event handlers attached to them directly.
      * This is all done here for neatness.
@@ -253,35 +257,37 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
      */
     this.attach_events = function(node, menu) {
         var container = node.ancestor('div'),
-            label = node.ancestor('label');
+            label = node.ancestor('label'),
+            marker = label.next('svg');
 
         // Function to collapse a comment to a marker icon.
         node.collapse = function(delay) {
             node.collapse.delay = Y.later(delay, node, function() {
-                container.addClass('commentcollapsed');
-            });
-        };
+                if (editor.collapsecomments && !this.is_menu_active()) {
+                    container.addClass('commentcollapsed');
+                }
+            }.bind(this));
+        }.bind(this);
 
         // Function to expand a comment.
         node.expand = function() {
-            container.removeClass('commentcollapsed');
+            if (node.getData('dragging') !== true) {
+                if (node.collapse.delay) {
+                    node.collapse.delay.cancel();
+                }
+                container.removeClass('commentcollapsed');
+            }
         };
 
         // Expand comment on mouse over (under certain conditions) or click/tap.
         container.on('mouseenter', function() {
             if (editor.currentedit.tool === 'comment' || editor.currentedit.tool === 'select' || this.editor.get('readonly')) {
                 node.expand();
-                if (node.collapse.delay) {
-                    node.collapse.delay.cancel();
-                }
             }
         }, this);
-        container.on('click', function() {
+        container.on('click|tap', function() {
             node.expand();
             node.focus();
-            if (node.collapse.delay) {
-                node.collapse.delay.cancel();
-            }
         }, this);
 
         // Functions to capture reverse tabbing events.
@@ -341,9 +347,7 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
         // Collapse comment on blur.
         container.on('blur', function() {
             node.active = false;
-            if (editor.collapsecomments) {
-                node.collapse(800);
-            }
+            node.collapse(800);
         }, this);
 
         if (!this.editor.get('readonly')) {
@@ -381,38 +385,37 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
             node.on('gesturemovestart', function(e) {
                 if (editor.currentedit.tool === 'select') {
                     e.preventDefault();
-                    node.setData('dragging', true);
-                    node.setData('offsetx', e.clientX - node.getX());
-                    node.setData('offsety', e.clientY - node.getY());
+                    if (editor.collapsecomments) {
+                        node.setData('offsetx', 8);
+                        node.setData('offsety', 8);
+                    } else {
+                        node.setData('offsetx', e.clientX - container.getX());
+                        node.setData('offsety', e.clientY - container.getY());
+                    }
                 }
             });
-            node.on('gesturemoveend', function() {
-                if (editor.currentedit.tool === 'select') {
-                    node.setData('dragging', false);
-                    this.editor.save_current_page();
-                }
-            }, null, this);
             node.on('gesturemove', function(e) {
                 if (editor.currentedit.tool === 'select') {
                     var x = e.clientX - node.getData('offsetx'),
                         y = e.clientY - node.getData('offsety'),
-                        nodewidth,
-                        nodeheight,
                         newlocation,
                         windowlocation,
                         bounds;
 
-                    nodewidth = parseInt(node.getStyle('width'), 10);
-                    nodeheight = parseInt(node.getStyle('height'), 10);
+                    if (node.getData('dragging') !== true) {
+                        // Collapse comment during move.
+                        node.collapse(0);
+                        node.setData('dragging', true);
+                    }
 
                     newlocation = this.editor.get_canvas_coordinates(new M.assignfeedback_editpdf.point(x, y));
                     bounds = this.editor.get_canvas_bounds(true);
                     bounds.x = 0;
                     bounds.y = 0;
 
-                    bounds.width -= nodewidth + 42;
-                    bounds.height -= nodeheight + 8;
-                    // Clip to the window size - the comment size.
+                    bounds.width -= 24;
+                    bounds.height -= 24;
+                    // Clip to the window size - the comment icon size.
                     newlocation.clip(bounds);
 
                     this.x = newlocation.x;
@@ -424,6 +427,63 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
                     this.drawable.store_position(container, windowlocation.x, windowlocation.y);
                 }
             }, null, this);
+            node.on('gesturemoveend', function() {
+                if (editor.currentedit.tool === 'select') {
+                    if (node.getData('dragging') === true) {
+                        node.setData('dragging', false);
+                    }
+                    this.editor.save_current_page();
+                }
+            }, null, this);
+            marker.on('gesturemovestart', function(e) {
+                if (editor.currentedit.tool === 'select') {
+                    e.preventDefault();
+                    node.setData('offsetx', e.clientX - container.getX());
+                    node.setData('offsety', e.clientY - container.getY());
+                    node.expand();
+                }
+            });
+            marker.on('gesturemove', function(e) {
+                if (editor.currentedit.tool === 'select') {
+                    var x = e.clientX - node.getData('offsetx'),
+                        y = e.clientY - node.getData('offsety'),
+                        newlocation,
+                        windowlocation,
+                        bounds;
+
+                    if (node.getData('dragging') !== true) {
+                        // Collapse comment during move.
+                        node.collapse(100);
+                        node.setData('dragging', true);
+                    }
+
+                    newlocation = this.editor.get_canvas_coordinates(new M.assignfeedback_editpdf.point(x, y));
+                    bounds = this.editor.get_canvas_bounds(true);
+                    bounds.x = 0;
+                    bounds.y = 0;
+
+                    bounds.width -= 24;
+                    bounds.height -= 24;
+                    // Clip to the window size - the comment icon size.
+                    newlocation.clip(bounds);
+
+                    this.x = newlocation.x;
+                    this.y = newlocation.y;
+
+                    windowlocation = this.editor.get_window_coordinates(newlocation);
+                    container.setX(windowlocation.x);
+                    container.setY(windowlocation.y);
+                    this.drawable.store_position(container, windowlocation.x, windowlocation.y);
+                }
+            }, null, this);
+            marker.on('gesturemoveend', function() {
+                if (editor.currentedit.tool === 'select') {
+                    if (node.getData('dragging') === true) {
+                        node.setData('dragging', false);
+                    }
+                    this.editor.save_current_page();
+                }
+            }, null, this);
 
             this.menu = new M.assignfeedback_editpdf.commentmenu({
                 buttonNode: this.menulink,
@@ -566,6 +626,23 @@ var COMMENT = function(editor, gradeid, pageno, x, y, width, colour, rawtext) {
         return (bounds.has_min_width() && bounds.has_min_height());
     };
 
+    /**
+     * Update comment position when rotating page.
+     * @public
+     * @method updatePosition
+     */
+    this.updatePosition = function() {
+        var node = this.drawable.nodes[0].one('textarea');
+        var container = node.ancestor('div');
+
+        var newlocation = new M.assignfeedback_editpdf.point(this.x, this.y);
+        var windowlocation = this.editor.get_window_coordinates(newlocation);
+
+        container.setX(windowlocation.x);
+        container.setY(windowlocation.y);
+        this.drawable.store_position(container, windowlocation.x, windowlocation.y);
+    };
+
 };
 
 M.assignfeedback_editpdf = M.assignfeedback_editpdf || {};
