@@ -24,7 +24,7 @@
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
-defined('MOODLE_INTERNAL') || die();
+use core_external\external_value;
 
 // File areas for file submission assignment.
 define('ASSIGNSUBMISSION_FILE_MAXSUMMARYFILES', 5);
@@ -323,8 +323,10 @@ class assign_submission_file extends assign_submission_plugin {
                                $submission->id);
 
         $currentsubmission = $this->get_file_submission($submission->id);
-        $currentsubmission->numfiles = 0;
-        $DB->update_record('assignsubmission_file', $currentsubmission);
+        if ($currentsubmission) {
+            $currentsubmission->numfiles = 0;
+            $DB->update_record('assignsubmission_file', $currentsubmission);
+        }
 
         return true;
     }
@@ -507,19 +509,6 @@ class assign_submission_file extends assign_submission_plugin {
         return true;
     }
 
-    /**
-     * Formatting for log info
-     *
-     * @param stdClass $submission The submission
-     * @return string
-     */
-    public function format_for_log(stdClass $submission) {
-        // Format the info for each submission plugin (will be added to log).
-        $filecount = $this->count_files($submission->id, ASSIGNSUBMISSION_FILE_FILEAREA);
-
-        return get_string('numfilesforlog', 'assignsubmission_file', $filecount);
-    }
-
     /**
      * Return true if there are no submission files
      * @param stdClass $submission
@@ -593,7 +582,7 @@ class assign_submission_file extends assign_submission_plugin {
     /**
      * Return a description of external params suitable for uploading a file submission from a webservice.
      *
-     * @return external_description|null
+     * @return \core_external\external_description|null
      */
     public function get_external_parameters() {
         return array(
@@ -637,4 +626,12 @@ class assign_submission_file extends assign_submission_plugin {
 
         return $sets;
     }
+
+    /**
+     * Determine if the plugin allows image file conversion
+     * @return bool
+     */
+    public function allow_image_conversion() {
+        return true;
+    }
 }
