@@ -29,7 +29,6 @@ defined('MOODLE_INTERNAL') || die();
 require_once($CFG->dirroot . '/mod/assign/locallib.php');
 
 use \core_privacy\local\metadata\collection;
-use \core_privacy\local\metadata\provider as metadataprovider;
 use \core_privacy\local\request\writer;
 use \core_privacy\local\request\contextlist;
 use \mod_assign\privacy\assign_plugin_request_data;
@@ -41,7 +40,10 @@ use \mod_assign\privacy\assign_plugin_request_data;
  * @copyright  2018 Adrian Greeve <adrian@moodle.com>
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class provider implements metadataprovider, \mod_assign\privacy\assignsubmission_provider {
+class provider implements
+        \core_privacy\local\metadata\provider,
+        \mod_assign\privacy\assignsubmission_provider,
+        \mod_assign\privacy\assignsubmission_user_provider {
 
     /**
      * Return meta data about this plugin.
@@ -79,6 +81,16 @@ class provider implements metadataprovider, \mod_assign\privacy\assignsubmission
         // No need.
     }
 
+    /**
+     * If you have tables that contain userids and you can generate entries in your tables without creating an
+     * entry in the assign_submission table then please fill in this method.
+     *
+     * @param  \core_privacy\local\request\userlist $userlist The userlist object
+     */
+    public static function get_userids_from_context(\core_privacy\local\request\userlist $userlist) {
+        // Not required.
+    }
+
     /**
      * Export all user data for this plugin.
      *
@@ -98,9 +110,10 @@ class provider implements metadataprovider, \mod_assign\privacy\assignsubmission
         $context = $exportdata->get_context();
         if (!empty($editortext)) {
             $submissiontext = new \stdClass();
-            $submissiontext->text = writer::with_context($context)->rewrite_pluginfile_urls([], '', '', '', $editortext);
             $currentpath = $exportdata->get_subcontext();
             $currentpath[] = get_string('privacy:path', 'assignsubmission_onlinetext');
+            $submissiontext->text = writer::with_context($context)->rewrite_pluginfile_urls($currentpath,
+                    'assignsubmission_onlinetext', 'submissions_onlinetext', $submission->id, $editortext);
             writer::with_context($context)
                     ->export_area_files($currentpath, 'assignsubmission_onlinetext', 'submissions_onlinetext', $submission->id)
                     // Add the text to the exporter.
@@ -135,7 +148,7 @@ class provider implements metadataprovider, \mod_assign\privacy\assignsubmission
                 ASSIGNSUBMISSION_ONLINETEXT_FILEAREA);
 
         // Delete the records in the table.
-        $DB->delete_records('assignsubmission_onlinetext', ['assignment' => $requestdata->get_assign()->get_instance()->id]);
+        $DB->delete_records('assignsubmission_onlinetext', ['assignment' => $requestdata->get_assignid()]);
     }
 
     /**
@@ -156,7 +169,33 @@ class provider implements metadataprovider, \mod_assign\privacy\assignsubmission
                 $submissionid);
 
         // Delete the records in the table.
-        $DB->delete_records('assignsubmission_onlinetext', ['assignment' => $deletedata->get_assign()->get_instance()->id,
+        $DB->delete_records('assignsubmission_onlinetext', ['assignment' => $deletedata->get_assignid(),
                 'submission' => $submissionid]);
     }
+
+    /**
+     * Deletes all submissions for the submission ids / userids provided in a context.
+     * assign_plugin_request_data contains:
+     * - context
+     * - assign object
+     * - submission ids (pluginids)
+     * - user ids
+     * @param  assign_plugin_request_data $deletedata A class that contains the relevant information required for deletion.
+     */
+    public static function delete_submissions(assign_plugin_request_data $deletedata) {
+        global $DB;
+
+        \core_plagiarism\privacy\provider::delete_plagiarism_for_users($deletedata->get_userids(), $deletedata->get_context());
+        if (empty($deletedata->get_submissionids())) {
+            return;
+        }
+
+        $fs = get_file_storage();
+        list($sql, $params) = $DB->get_in_or_equal($deletedata->get_submissionids(), SQL_PARAMS_NAMED);
+        $fs->delete_area_files_select($deletedata->get_context()->id,
+                'assignsubmission_onlinetext', ASSIGNSUBMISSION_ONLINETEXT_FILEAREA, $sql, $params);
+
+        $params['assignid'] = $deletedata->get_assignid();
+        $DB->delete_records_select('assignsubmission_onlinetext', "assignment = :assignid AND submission $sql", $params);
+    }
 }
