@@ -16,6 +16,8 @@
 
 namespace mod_data;
 
+use stdClass;
+
 /**
  * PHPUnit data generator testcase.
  *
@@ -118,16 +120,6 @@ class generator_test extends \advanced_testcase {
         }
 
         $this->assertEquals(count($fieldtypes), $DB->count_records('data_fields', ['dataid' => $data->id]));
-
-        $addtemplate = $DB->get_record('data', ['id' => $data->id], 'addtemplate');
-        $addtemplate = $addtemplate->addtemplate;
-
-        for ($i = 1; $i < $count; $i++) {
-            $fieldname = 'field-' . $i;
-            $position = strpos($addtemplate, '[[' . $fieldname . ']]');
-            $this->assertIsNumeric($position);
-            $this->assertGreaterThanOrEqual(0, $position);
-        }
     }
 
     /**
@@ -247,4 +239,112 @@ class generator_test extends \advanced_testcase {
             array_values(\core_tag_tag::get_item_tags_array('mod_data', 'data_records', $datarecordid))
         );
     }
+
+    /**
+     * Test for create_preset().
+     *
+     * @dataProvider create_preset_provider
+     * @covers ::create_preset
+     * @param stdClass|null $record data for the preset that will be created (like name or description)
+     */
+    public function test_create_preset(?stdClass $record) {
+        global $USER;
+
+        $this->resetAfterTest();
+        $this->setAdminUser();
+
+        $course = $this->getDataGenerator()->create_course();
+        $activity = $this->getDataGenerator()->create_module(manager::MODULE, ['course' => $course]);
+        $cm = get_coursemodule_from_id(manager::MODULE, $activity->cmid, 0, false, MUST_EXIST);
+        if (!is_null($record) && property_exists($record, 'user')) {
+            $user = $this->getDataGenerator()->create_and_enrol($course, 'teacher', (object) ['username' => $record->user]);
+            $record->userid = $user->id;
+            unset($record->user);
+        }
+
+        // Check initially there are no saved presets.
+        $manager = manager::create_from_coursemodule($cm);
+        $savedpresets = $manager->get_available_saved_presets();
+        $this->assertEmpty($savedpresets);
+
+        // Create one preset with the configuration in $record.
+        $plugingenerator = $this->getDataGenerator()->get_plugin_generator('mod_data');
+        $preset = $plugingenerator->create_preset($activity, $record);
+        // Check the preset has been saved.
+        $savedpresets = $manager->get_available_saved_presets();
+        $this->assertCount(1, $savedpresets);
+        // Check the preset name has the expected value.
+        if (is_null($record) || !property_exists($record, 'name')) {
+            $this->assertStringStartsWith('New preset', $preset->name);
+        } else {
+            $this->assertEquals($record->name, $preset->name);
+        }
+        // Check the preset description has the expected value.
+        if (is_null($record) || !property_exists($record, 'description')) {
+            $this->assertEmpty($preset->description);
+        } else {
+            $this->assertEquals($record->description, $preset->description);
+        }
+        // Check the preset author has the expected value.
+        if (is_null($record) || !property_exists($record, 'userid')) {
+            $this->assertEquals($USER->id, $preset->get_userid());
+        } else {
+            $this->assertEquals($record->userid, $preset->get_userid());
+        }
+        // Check the file has been updated properly.
+        $this->assertNotNull($preset->storedfile);
+    }
+
+    /**
+     * Data provider for test_create_preset().
+     *
+     * @return array
+     */
+    public function create_preset_provider(): array {
+        return [
+            'Create using the default configuration' => [
+                'record' => null,
+            ],
+            'Create with a given name but no description' => [
+                'record' => (object) [
+                    'name' => 'World recipes preset',
+                ],
+            ],
+            'Create with a given description but no name' => [
+                'record' => (object) [
+                    'description' => 'This is a preset to collect the most popular world recipes.',
+                ],
+            ],
+            'Create with a given name and description' => [
+                'record' => (object) [
+                    'name' => 'World recipes preset',
+                    'description' => 'This is a preset to collect the most popular world recipes.',
+                ],
+            ],
+            'Create with a given user but no description or name' => [
+                'record' => (object) [
+                    'user' => 'teacher1',
+                ],
+            ],
+            'Create with a given name and user but no description' => [
+                'record' => (object) [
+                    'name' => 'World recipes preset',
+                    'user' => 'teacher1',
+                ],
+            ],
+            'Create with a given description and user but no name' => [
+                'record' => (object) [
+                    'description' => 'This is a preset to collect the most popular world recipes.',
+                    'user' => 'teacher1',
+                ],
+            ],
+            'Create with a given name, description and user' => [
+                'record' => (object) [
+                    'name' => 'World recipes preset',
+                    'description' => 'This is a preset to collect the most popular world recipes.',
+                    'user' => 'teacher1',
+                ],
+            ],
+        ];
+    }
 }
