@@ -14,21 +14,18 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-/**
- * The capability manager tests.
- *
- * @package    mod_forum
- * @copyright  2019 Ryan Wyllie <ryan@moodle.com>
- * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
+namespace mod_forum;
+
+use mod_forum\local\container;
+use mod_forum\local\entities\forum;
+use mod_forum\local\managers\capability as capability_manager;
+use mod_forum_tests_generator_trait;
 
 defined('MOODLE_INTERNAL') || die();
 
 global $CFG;
 require_once(__DIR__ . '/generator_trait.php');
 
-use mod_forum\local\managers\capability as capability_manager;
-
 /**
  * The capability manager tests.
  *
@@ -37,7 +34,7 @@ use mod_forum\local\managers\capability as capability_manager;
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  * @coversDefaultClass \mod_forum\local\managers\capability
  */
-class mod_forum_managers_capability_testcase extends advanced_testcase {
+class managers_capability_test extends \advanced_testcase {
     // Make use of the test generator trait.
     use mod_forum_tests_generator_trait;
 
@@ -80,7 +77,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
     /**
      * Setup function before each test.
      */
-    public function setUp() {
+    public function setUp(): void {
         global $DB;
 
         // We must clear the subscription caches. This has to be done both before each test, and after in case of other
@@ -89,19 +86,19 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
 
         $datagenerator = $this->getDataGenerator();
         $this->user = $datagenerator->create_user();
-        $this->managerfactory = \mod_forum\local\container::get_manager_factory();
-        $this->entityfactory = \mod_forum\local\container::get_entity_factory();
+        $this->managerfactory = container::get_manager_factory();
+        $this->entityfactory = container::get_entity_factory();
         $this->course = $datagenerator->create_course();
         $this->forumrecord = $datagenerator->create_module('forum', ['course' => $this->course->id]);
         $this->coursemodule = get_coursemodule_from_instance('forum', $this->forumrecord->id);
-        $this->context = context_module::instance($this->coursemodule->id);
+        $this->context = \context_module::instance($this->coursemodule->id);
         $this->roleid = $DB->get_field('role', 'id', ['shortname' => 'teacher'], MUST_EXIST);
 
         $datagenerator->enrol_user($this->user->id, $this->course->id, 'teacher');
         [$discussion, $post] = $this->helper_post_to_forum($this->forumrecord, $this->user, ['timemodified' => time() - 100]);
-        $this->discussion = $this->entityfactory->get_discussion_from_stdclass($discussion);
+        $this->discussion = $this->entityfactory->get_discussion_from_stdClass($discussion);
         $this->discussionrecord = $discussion;
-        $this->post = $this->entityfactory->get_post_from_stdclass(
+        $this->post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $post, ['timecreated' => time() - 100])
         );
         $this->postrecord = $post;
@@ -112,7 +109,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
     /**
      * Tear down function after each test.
      */
-    public function tearDown() {
+    public function tearDown(): void {
         // We must clear the subscription caches. This has to be done both before each test, and after in case of other
         // tests using these functions.
         \mod_forum\subscriptions::reset_forum_cache();
@@ -122,11 +119,11 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
      * Helper function to create a forum entity.
      *
      * @param array $forumproperties List of properties to override the prebuilt forum
-     * @return forum_entity
+     * @return forum
      */
     private function create_forum(array $forumproperties = []) {
         $forumrecord = (object) array_merge((array) $this->forumrecord, $forumproperties);
-        return $this->entityfactory->get_forum_from_stdclass(
+        return $this->entityfactory->get_forum_from_stdClass(
             $forumrecord,
             $this->context,
             $this->coursemodule,
@@ -217,8 +214,8 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
             ['course' => $this->course->id, 'groupmode' => SEPARATEGROUPS]
         );
         $coursemodule = get_coursemodule_from_instance('forum', $forumrecord->id);
-        $context = context_module::instance($coursemodule->id);
-        $forum = $this->entityfactory->get_forum_from_stdclass(
+        $context = \context_module::instance($coursemodule->id);
+        $forum = $this->entityfactory->get_forum_from_stdClass(
             $forumrecord,
             $context,
             $coursemodule,
@@ -239,6 +236,18 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->getDataGenerator()->create_group_member(['userid' => $user->id, 'groupid' => $group->id]);
 
         $this->assertTrue($capabilitymanager->can_create_discussions($user, $group->id));
+
+        // Test if cut off date is reached.
+        $now = time();
+        $forum = $this->create_forum(['cutoffdate' => $now + 86400 , 'blockafter' => 5, 'blockperiod' => 86400]);
+        $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
+        $this->prevent_capability('mod/forum:postwithoutthrottling');
+        $this->assertTrue($capabilitymanager->can_create_discussions($user));
+
+        $forum = $this->create_forum(['cutoffdate' => $now + 86400 , 'blockafter' => 1, 'blockperiod' => 86400]);
+        $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
+        $this->prevent_capability('mod/forum:postwithoutthrottling');
+        $this->assertFalse($capabilitymanager->can_create_discussions($user));
     }
 
     /**
@@ -408,7 +417,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $forum = $this->create_forum();
         $user = $this->user;
         $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
-        $cache = cache::make('mod_forum', 'forum_is_tracked');
+        $cache = \cache::make('mod_forum', 'forum_is_tracked');
 
         $user->trackforums = true;
         $prefid = $DB->insert_record('forum_track_prefs', ['userid' => $user->id, 'forumid' => $forum->get_id()]);
@@ -564,8 +573,8 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
             ['course' => $this->course->id, 'groupmode' => SEPARATEGROUPS]
         );
         $coursemodule = get_coursemodule_from_instance('forum', $forumrecord->id);
-        $context = context_module::instance($coursemodule->id);
-        $forum = $this->entityfactory->get_forum_from_stdclass(
+        $context = \context_module::instance($coursemodule->id);
+        $forum = $this->entityfactory->get_forum_from_stdClass(
             $forumrecord,
             $context,
             $coursemodule,
@@ -580,7 +589,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertFalse($capabilitymanager->can_post_in_discussion($user, $discussion));
 
         $group = $this->getDataGenerator()->create_group(['courseid' => $this->course->id]);
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['groupid' => $group->id])
         );
 
@@ -596,8 +605,8 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
             ['course' => $this->course->id, 'groupmode' => VISIBLEGROUPS]
         );
         $coursemodule = get_coursemodule_from_instance('forum', $forumrecord->id);
-        $context = context_module::instance($coursemodule->id);
-        $forum = $this->entityfactory->get_forum_from_stdclass(
+        $context = \context_module::instance($coursemodule->id);
+        $forum = $this->entityfactory->get_forum_from_stdClass(
             $forumrecord,
             $context,
             $coursemodule,
@@ -612,7 +621,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertTrue($capabilitymanager->can_post_in_discussion($user, $discussion));
 
         $group = $this->getDataGenerator()->create_group(['courseid' => $this->course->id]);
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['groupid' => $group->id])
         );
 
@@ -621,6 +630,18 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->getDataGenerator()->create_group_member(['userid' => $user->id, 'groupid' => $group->id]);
 
         $this->assertTrue($capabilitymanager->can_post_in_discussion($user, $discussion));
+
+        $now = time();
+        $forum = $this->create_forum(['cutoffdate' => $now + 86400 , 'blockafter' => 5, 'blockperiod' => 86400]);
+        $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
+        $this->prevent_capability('mod/forum:postwithoutthrottling');
+        $this->give_capability('mod/forum:replypost');
+        $this->assertTrue($capabilitymanager->can_post_in_discussion($user, $discussion));
+
+        $forum = $this->create_forum(['cutoffdate' => $now + 86400 , 'blockafter' => 1, 'blockperiod' => 86400]);
+        $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
+        $this->prevent_capability('mod/forum:postwithoutthrottling');
+        $this->assertFalse($capabilitymanager->can_post_in_discussion($user, $discussion));
     }
 
     /**
@@ -647,6 +668,18 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $CFG->maxeditingtime = 200;
         $this->assertTrue($capabilitymanager->can_edit_post($user, $discussion, $post));
 
+        // Can not edit within editing time if $post->mailnow > 0 (selected).
+        $CFG->maxeditingtime = 200;
+        $post = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, ['mailnow' => 1])
+        );
+        $this->assertFalse($capabilitymanager->can_edit_post($user, $discussion, $post));
+
+        // Back to normal - mailnow not selected.
+        $post = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, ['mailnow' => 0])
+        );
+
         // 10 seconds to edit. No longer in editing time.
         $CFG->maxeditingtime = 10;
         $this->assertFalse($capabilitymanager->can_edit_post($user, $discussion, $post));
@@ -664,7 +697,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $forum = $this->create_forum(['type' => 'news']);
         $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
         // Discussion hasn't started yet.
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['timestart' => time() + 100])
         );
 
@@ -673,7 +706,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         // Back to a discussion that has started.
         $discussion = $this->discussion;
         // Post is a reply.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['parent' => 5])
         );
 
@@ -689,7 +722,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
 
         // Create a new post that definitely isn't the first post of the discussion.
         // Only the author, and a user with editanypost can edit it.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['id' => $post->get_id() + 100])
         );
         $this->give_capability('mod/forum:editanypost');
@@ -698,7 +731,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
 
         $post = $this->post;
         // Set the first post of the discussion to our post.
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['firstpost' => $post->get_id()])
         );
 
@@ -727,7 +760,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
 
         // Set the first post of the discussion to our post.
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['firstpost' => $post->get_id()])
         );
 
@@ -735,7 +768,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertFalse($capabilitymanager->can_delete_post($user, $discussion, $post));
 
         // Set the first post of the discussion to something else.
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['firstpost' => $post->get_id() - 1])
         );
 
@@ -748,8 +781,19 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         // 200 second editing time to make sure our post is still within it.
         $CFG->maxeditingtime = 200;
 
+        // Can not delete within editing time if $post->mailnow > 0 (selected).
+        $post = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, ['mailnow' => 1])
+        );
+        $this->assertFalse($capabilitymanager->can_delete_post($user, $discussion, $post));
+
+        // Back to normal - mailnow not selected.
+        $post = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, ['mailnow' => 0])
+        );
+
         // Make the post owned by someone else.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['userid' => $user->id - 1])
         );
 
@@ -790,7 +834,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
 
         // Make the post a reply.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['parent' => 5])
         );
 
@@ -801,7 +845,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertTrue($capabilitymanager->can_split_post($user, $discussion, $post));
 
         // Make the post have no parent.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['parent' => 0])
         );
 
@@ -810,7 +854,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $forum = $this->create_forum(['type' => 'single']);
         $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
         // Make the post a reply.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['parent' => 5])
         );
 
@@ -818,7 +862,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertFalse($capabilitymanager->can_split_post($user, $discussion, $post));
 
         // Make the post a private reply.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['parent' => 5, 'privatereplyto' => $user->id])
         );
 
@@ -874,8 +918,8 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
             ['course' => $this->course->id, 'groupmode' => SEPARATEGROUPS]
         );
         $coursemodule = get_coursemodule_from_instance('forum', $forumrecord->id);
-        $context = context_module::instance($coursemodule->id);
-        $forum = $this->entityfactory->get_forum_from_stdclass(
+        $context = \context_module::instance($coursemodule->id);
+        $forum = $this->entityfactory->get_forum_from_stdClass(
             $forumrecord,
             $context,
             $coursemodule,
@@ -890,7 +934,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertFalse($capabilitymanager->can_reply_to_post($user, $discussion, $post));
 
         $group = $this->getDataGenerator()->create_group(['courseid' => $this->course->id]);
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['groupid' => $group->id])
         );
 
@@ -906,8 +950,8 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
             ['course' => $this->course->id, 'groupmode' => VISIBLEGROUPS]
         );
         $coursemodule = get_coursemodule_from_instance('forum', $forumrecord->id);
-        $context = context_module::instance($coursemodule->id);
-        $forum = $this->entityfactory->get_forum_from_stdclass(
+        $context = \context_module::instance($coursemodule->id);
+        $forum = $this->entityfactory->get_forum_from_stdClass(
             $forumrecord,
             $context,
             $coursemodule,
@@ -922,7 +966,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertTrue($capabilitymanager->can_reply_to_post($user, $discussion, $post));
 
         $group = $this->getDataGenerator()->create_group(['courseid' => $this->course->id]);
-        $discussion = $this->entityfactory->get_discussion_from_stdclass(
+        $discussion = $this->entityfactory->get_discussion_from_stdClass(
             (object) array_merge((array) $this->discussionrecord, ['groupid' => $group->id])
         );
 
@@ -933,7 +977,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertTrue($capabilitymanager->can_reply_to_post($user, $discussion, $post));
 
         // Make the post a private reply.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['parent' => 5, 'privatereplyto' => $user->id])
         );
 
@@ -941,6 +985,52 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertFalse($capabilitymanager->can_reply_to_post($user, $discussion, $post));
     }
 
+    /**
+     * Test for \mod_forum\local\managers\capability::can_reply_to_post() involving Q & A forums.
+     */
+    public function test_can_reply_to_post_in_qanda_forum() {
+        global $CFG;
+
+        $this->resetAfterTest();
+
+        // Set max editing time to 10 seconds.
+        $CFG->maxeditingtime = 10;
+
+        $qandaforum = $this->create_forum(['type' => 'qanda']);
+        $datagenerator = $this->getDataGenerator();
+        $capabilitymanager = $this->managerfactory->get_capability_manager($qandaforum);
+
+        // Student 1.
+        $student1 = $datagenerator->create_user(['firstname' => 'S1']);
+        $datagenerator->enrol_user($student1->id, $this->course->id, 'student');
+        // Confirm Student 1 can reply to the question.
+        $this->assertTrue($capabilitymanager->can_reply_to_post($student1, $this->discussion, $this->post));
+
+        // Student 2.
+        $student2 = $datagenerator->create_user(['firstname' => 'S2']);
+        $datagenerator->enrol_user($student2->id, $this->course->id, 'student');
+        // Confirm Student 2 can reply to the question.
+        $this->assertTrue($capabilitymanager->can_reply_to_post($student2, $this->discussion, $this->post));
+
+        // Reply to the question as student 1.
+        $now = time();
+        $options = ['parent' => $this->post->get_id(), 'created' => $now - 100];
+        $student1post = $this->helper_post_to_discussion($this->forumrecord, $this->discussionrecord, $student1, $options);
+        $student1postentity = $this->entityfactory->get_post_from_stdClass($student1post);
+
+        // Confirm Student 2 cannot reply student 1's answer yet.
+        $this->assertFalse($capabilitymanager->can_reply_to_post($student2, $this->discussion, $student1postentity));
+
+        // Reply to the question as student 2.
+        $this->helper_post_to_discussion($this->forumrecord, $this->discussionrecord, $student2, $options);
+
+        // Reinitialise capability manager first to ensure we don't return cached values.
+        $capabilitymanager = $this->managerfactory->get_capability_manager($qandaforum);
+
+        // Confirm Student 2 can now reply to student 1's answer.
+        $this->assertTrue($capabilitymanager->can_reply_to_post($student2, $this->discussion, $student1postentity));
+    }
+
     /**
      * Ensure that can_reply_privately_to_post works as expected.
      *
@@ -964,7 +1054,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->assertTrue($capabilitymanager->can_reply_privately_to_post($this->user, $post));
 
         // Make the post a private reply.
-        $post = $this->entityfactory->get_post_from_stdclass(
+        $post = $this->entityfactory->get_post_from_stdClass(
             (object) array_merge((array) $this->postrecord, ['parent' => 5, 'privatereplyto' => $user->id])
         );
 
@@ -989,8 +1079,9 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $discussion = $this->discussion;
         $post = $this->post;
 
-        $privatepost = $this->entityfactory->get_post_from_stdclass(
-            (object) array_merge((array) $this->postrecord, ['parent' => $post->get_id(), 'privatereplyto' => $otheruser->id])
+        $postproperties = ['parent' => $post->get_id(), 'userid' => $otheruser->id, 'privatereplyto' => $otheruser->id];
+        $privatepost = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, $postproperties)
         );
 
         $this->prevent_capability('mod/forum:readprivatereplies');
@@ -1013,23 +1104,41 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
 
         $discussion = $this->discussion;
         $post = $this->post;
-        $privatepost = $this->entityfactory->get_post_from_stdclass(
-            (object) array_merge((array) $this->postrecord, ['parent' => $post->get_id(), 'privatereplyto' => $otheruser->id])
-        );
-        $privateposttome = $this->entityfactory->get_post_from_stdclass(
-            (object) array_merge((array) $this->postrecord, ['parent' => $post->get_id(), 'privatereplyto' => $user->id])
+
+        $postproperties = ['parent' => $post->get_id(), 'userid' => $user->id, 'privatereplyto' => $user->id];
+        $privatepostfrommetome = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, $postproperties)
         );
 
-        // Can always view public replies, and those to me.
+        $postproperties = ['parent' => $post->get_id(), 'userid' => $user->id, 'privatereplyto' => $otheruser->id];
+        $privatepostfrommetoother = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, $postproperties)
+        );
+
+        $postproperties = ['parent' => $post->get_id(), 'userid' => $otheruser->id, 'privatereplyto' => $user->id];
+        $privatepostfromothertome = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, $postproperties)
+        );
+
+        $postproperties = ['parent' => $post->get_id(), 'userid' => $otheruser->id, 'privatereplyto' => $otheruser->id];
+        $privatepostfromothertoother = $this->entityfactory->get_post_from_stdClass(
+            (object) array_merge((array) $this->postrecord, $postproperties)
+        );
+
+        // Can always view public replies, and private replies by me or to me.
         $this->prevent_capability('mod/forum:readprivatereplies');
         $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $post));
-        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privateposttome));
-        $this->assertFalse($capabilitymanager->can_view_post_shell($this->user, $privatepost));
+        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepostfrommetome));
+        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepostfrommetoother));
+        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepostfromothertome));
+        $this->assertFalse($capabilitymanager->can_view_post_shell($this->user, $privatepostfromothertoother));
 
         $this->give_capability('mod/forum:readprivatereplies');
         $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $post));
-        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privateposttome));
-        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepost));
+        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepostfrommetome));
+        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepostfrommetoother));
+        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepostfromothertome));
+        $this->assertTrue($capabilitymanager->can_view_post_shell($this->user, $privatepostfromothertoother));
     }
 
     /**
@@ -1161,7 +1270,7 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $forum = $this->create_forum();
         $user = $this->user;
         $capabilitymanager = $this->managerfactory->get_capability_manager($forum);
-        $context = context_system::instance();
+        $context = \context_system::instance();
         $roleid = $DB->get_field('role', 'id', ['shortname' => 'user'], MUST_EXIST);
 
         assign_capability('moodle/tag:manage', CAP_PREVENT, $roleid, $context->id, true);
@@ -1187,4 +1296,98 @@ class mod_forum_managers_capability_testcase extends advanced_testcase {
         $this->prevent_capability('mod/forum:readprivatereplies');
         $this->assertFalse($capabilitymanager->can_view_any_private_reply($this->user));
     }
+
+
+    /**
+     * Test delete a post with ratings.
+     */
+    public function test_validate_delete_post_with_ratings() {
+        global $DB;
+        $this->resetAfterTest(true);
+
+        // Setup test data.
+        $course = $this->getDataGenerator()->create_course();
+        $forum = $this->getDataGenerator()->create_module('forum', array('course' => $course->id));
+        $user = $this->getDataGenerator()->create_user();
+        $role = $DB->get_record('role', array('shortname' => 'student'), '*', MUST_EXIST);
+        self::getDataGenerator()->enrol_user($user->id, $course->id, $role->id);
+
+        // Add a discussion.
+        $record = new \stdClass();
+        $record->course = $course->id;
+        $record->userid = $user->id;
+        $record->forum = $forum->id;
+        $record->created =
+        $discussion = $this->getDataGenerator()->get_plugin_generator('mod_forum')->create_discussion($record);
+
+        // Add rating.
+        $post = $DB->get_record('forum_posts', array('discussion' => $discussion->id));
+        $post->totalscore = 80;
+        $DB->update_record('forum_posts', $post);
+
+        $vaultfactory = container::get_vault_factory();
+        $forumvault = $vaultfactory->get_forum_vault();
+        $discussionvault = $vaultfactory->get_discussion_vault();
+        $postvault = $vaultfactory->get_post_vault();
+
+        $postentity = $postvault->get_from_id($post->id);
+        $discussionentity = $discussionvault->get_from_id($postentity->get_discussion_id());
+        $forumentity = $forumvault->get_from_id($discussionentity->get_forum_id());
+        $capabilitymanager = $this->managerfactory->get_capability_manager($forumentity);
+
+        $this->setUser($user);
+        $this->expectExceptionMessage(get_string('couldnotdeleteratings', 'rating'));
+        $capabilitymanager->validate_delete_post($user, $discussionentity, $postentity, false);
+    }
+
+    /**
+     * Test delete a post with replies.
+     */
+    public function test_validate_delete_post_with_replies() {
+        global $DB;
+        $this->resetAfterTest(true);
+
+        // Setup test data.
+        $course = $this->getDataGenerator()->create_course();
+        $forum = $this->getDataGenerator()->create_module('forum', array('course' => $course->id));
+        $user = $this->getDataGenerator()->create_user();
+        $role = $DB->get_record('role', array('shortname' => 'student'), '*', MUST_EXIST);
+        self::getDataGenerator()->enrol_user($user->id, $course->id, $role->id);
+
+        // Add a discussion.
+        $record = new \stdClass();
+        $record->course = $course->id;
+        $record->userid = $user->id;
+        $record->forum = $forum->id;
+        $record->created =
+        $discussion = $this->getDataGenerator()->get_plugin_generator('mod_forum')->create_discussion($record);
+
+        $parentpost = $DB->get_record('forum_posts', array('discussion' => $discussion->id));
+        // Add a post.
+        $record = new \stdClass();
+        $record->course = $course->id;
+        $record->userid = $user->id;
+        $record->forum = $forum->id;
+        $record->discussion = $discussion->id;
+        $record->parent = $parentpost->id;
+        $this->getDataGenerator()->get_plugin_generator('mod_forum')->create_post($record);
+
+        $vaultfactory = container::get_vault_factory();
+        $forumvault = $vaultfactory->get_forum_vault();
+        $discussionvault = $vaultfactory->get_discussion_vault();
+        $postvault = $vaultfactory->get_post_vault();
+
+        $postentity = $postvault->get_from_id($parentpost->id);
+        $discussionentity = $discussionvault->get_from_id($postentity->get_discussion_id());
+        $forumentity = $forumvault->get_from_id($discussionentity->get_forum_id());
+        $capabilitymanager = $this->managerfactory->get_capability_manager($forumentity);
+
+        $this->setUser($user);
+        // Get reply count.
+        $replycount = $postvault->get_reply_count_for_post_id_in_discussion_id(
+            $user, $postentity->get_id(), $discussionentity->get_id(), true);
+        $this->expectExceptionMessage(get_string('couldnotdeletereplies', 'forum'));
+        $capabilitymanager->validate_delete_post($user, $discussionentity, $postentity, $replycount);
+    }
+
 }
