@@ -14,20 +14,13 @@
 // You should have received a copy of the GNU General Public License
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
-/**
- * This file defines the quiz manual grading report class.
- *
- * @package   quiz_grading
- * @copyright 2006 Gustav Delius
- * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
- */
-
+use mod_quiz\local\reports\report_base;
+use mod_quiz\quiz_attempt;
 
 defined('MOODLE_INTERNAL') || die();
 
 require_once($CFG->dirroot . '/mod/quiz/report/grading/gradingsettings_form.php');
 
-
 /**
  * Quiz report to help teachers manually grade questions that need it.
  *
@@ -38,7 +31,7 @@ require_once($CFG->dirroot . '/mod/quiz/report/grading/gradingsettings_form.php'
  * @copyright 2006 Gustav Delius
  * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class quiz_grading_report extends quiz_default_report {
+class quiz_grading_report extends report_base {
     const DEFAULT_PAGE_SIZE = 5;
     const DEFAULT_ORDER = 'random';
 
@@ -72,6 +65,9 @@ class quiz_grading_report extends quiz_default_report {
     /** @var string fragment of SQL code to restrict to the relevant users. */
     protected $userssql;
 
+    /** @var array extra user fields. */
+    protected $extrauserfields = [];
+
     public function display($quiz, $cm, $course) {
 
         $this->quiz = $quiz;
@@ -84,7 +80,7 @@ class quiz_grading_report extends quiz_default_report {
         $grade = optional_param('grade', null, PARAM_ALPHA);
 
         $includeauto = optional_param('includeauto', false, PARAM_BOOL);
-        if (!in_array($grade, array('all', 'needsgrading', 'autograded', 'manuallygraded'))) {
+        if (!in_array($grade, ['all', 'needsgrading', 'autograded', 'manuallygraded'])) {
             $grade = null;
         }
         $pagesize = optional_param('pagesize',
@@ -93,10 +89,10 @@ class quiz_grading_report extends quiz_default_report {
         $page = optional_param('page', 0, PARAM_INT);
         $order = optional_param('order',
                 get_user_preferences('quiz_grading_order', self::DEFAULT_ORDER),
-                PARAM_ALPHA);
+                PARAM_ALPHAEXT);
 
         // Assemble the options required to reload this page.
-        $optparams = array('includeauto', 'page');
+        $optparams = ['includeauto', 'page'];
         foreach ($optparams as $param) {
             if ($$param) {
                 $this->viewoptions[$param] = $$param;
@@ -117,14 +113,20 @@ class quiz_grading_report extends quiz_default_report {
         $this->context = context_module::instance($this->cm->id);
         require_capability('mod/quiz:grade', $this->context);
         $shownames = has_capability('quiz/grading:viewstudentnames', $this->context);
-        $showidnumbers = has_capability('quiz/grading:viewidnumber', $this->context);
-
+        // Whether the current user can see custom user fields.
+        $showcustomfields = has_capability('quiz/grading:viewidnumber', $this->context);
+        $userfieldsapi = \core_user\fields::for_identity($this->context)->with_name();
+        $customfields = [];
+        foreach ($userfieldsapi->get_required_fields([\core_user\fields::PURPOSE_IDENTITY]) as $field) {
+            $customfields[] = $field;
+        }
         // Validate order.
-        if (!in_array($order, array('random', 'date', 'studentfirstname', 'studentlastname', 'idnumber'))) {
+        $orderoptions = array_merge(['random', 'date', 'studentfirstname', 'studentlastname'], $customfields);
+        if (!in_array($order, $orderoptions)) {
             $order = self::DEFAULT_ORDER;
         } else if (!$shownames && ($order == 'studentfirstname' || $order == 'studentlastname')) {
             $order = self::DEFAULT_ORDER;
-        } else if (!$showidnumbers && $order == 'idnumber') {
+        } else if (!$showcustomfields && in_array($order, $customfields)) {
             $order = self::DEFAULT_ORDER;
         }
         if ($order == 'random') {
@@ -139,18 +141,25 @@ class quiz_grading_report extends quiz_default_report {
 
         // Process any submitted data.
         if ($data = data_submitted() && confirm_sesskey() && $this->validate_submitted_marks()) {
-            $this->process_submitted_data();
+            // Changes done to handle attempts being missed from grading due to redirecting to new page.
+            $attemptsgraded = $this->process_submitted_data();
 
-            redirect($this->grade_question_url($slot, $questionid, $grade, $page + 1));
+            $nextpagenumber = $page + 1;
+            // If attempts need grading and one or more have now been graded, then page number should remain the same.
+            if ($grade == 'needsgrading' && $attemptsgraded) {
+                $nextpagenumber = $page;
+            }
+
+            redirect($this->grade_question_url($slot, $questionid, $grade, $nextpagenumber));
         }
 
         // Get the group, and the list of significant users.
         $this->currentgroup = $this->get_current_group($cm, $course, $this->context);
         if ($this->currentgroup == self::NO_GROUPS_ALLOWED) {
-            $this->userssql = array();
+            $this->userssql = [];
         } else {
             $this->userssql = get_enrolled_sql($this->context,
-                    array('mod/quiz:reviewmyattempts', 'mod/quiz:attempt'), $this->currentgroup);
+                    ['mod/quiz:reviewmyattempts', 'mod/quiz:attempt'], $this->currentgroup);
         }
 
         $hasquestions = quiz_has_questions($this->quiz->id);
@@ -183,7 +192,7 @@ class quiz_grading_report extends quiz_default_report {
         }
 
         $this->display_grading_interface($slot, $questionid, $grade,
-                $pagesize, $page, $shownames, $showidnumbers, $order, $counts);
+                $pagesize, $page, $shownames, $showcustomfields, $order, $counts);
         return true;
     }
 
@@ -197,15 +206,15 @@ class quiz_grading_report extends quiz_default_report {
         $where = "quiza.quiz = :mangrquizid AND
                 quiza.preview = 0 AND
                 quiza.state = :statefinished";
-        $params = array('mangrquizid' => $this->cm->instance, 'statefinished' => quiz_attempt::FINISHED);
+        $params = ['mangrquizid' => $this->cm->instance, 'statefinished' => quiz_attempt::FINISHED];
 
         $usersjoin = '';
         $currentgroup = groups_get_activity_group($this->cm, true);
         $enrolleduserscount = count_enrolled_users($this->context,
-                array('mod/quiz:reviewmyattempts', 'mod/quiz:attempt'), $currentgroup);
+                ['mod/quiz:reviewmyattempts', 'mod/quiz:attempt'], $currentgroup);
         if ($currentgroup) {
             $userssql = get_enrolled_sql($this->context,
-                    array('mod/quiz:reviewmyattempts', 'mod/quiz:attempt'), $currentgroup);
+                    ['mod/quiz:reviewmyattempts', 'mod/quiz:attempt'], $currentgroup);
             if ($enrolleduserscount < 1) {
                 $where .= ' AND quiza.userid = 0';
             } else {
@@ -230,17 +239,23 @@ class quiz_grading_report extends quiz_default_report {
         $params[] = quiz_attempt::FINISHED;
         $params[] = $this->quiz->id;
 
-        $fields = 'quiza.*, u.idnumber, ';
-        $userfieldsapi = \core_user\fields::for_name();
-        $fields .= $userfieldsapi->get_sql('u', false, '', '', false)->selects;
+        $fields = 'quiza.*, ';
+        $userfieldsapi = \core_user\fields::for_identity($this->context)->with_name();
+        $userfieldssql = $userfieldsapi->get_sql('u', false, '', 'userid', false);
+        $fields .= $userfieldssql->selects;
+        foreach ($userfieldsapi->get_required_fields([\core_user\fields::PURPOSE_IDENTITY]) as $userfield) {
+            $this->extrauserfields[] = s($userfield);
+        }
+        $params = array_merge($userfieldssql->params, $params);
         $attemptsbyid = $DB->get_records_sql("
                 SELECT $fields
                 FROM {quiz_attempts} quiza
                 JOIN {user} u ON u.id = quiza.userid
+                {$userfieldssql->joins}
                 WHERE quiza.uniqueid $asql AND quiza.state = ? AND quiza.quiz = ?",
                 $params);
 
-        $attempts = array();
+        $attempts = [];
         foreach ($attemptsbyid as $attempt) {
             $attempts[$attempt->uniqueid] = $attempt;
         }
@@ -369,7 +384,7 @@ class quiz_grading_report extends quiz_default_report {
 
             $row[] = $this->questions[$counts->slot]->number;
 
-            $row[] = $PAGE->get_renderer('question', 'bank')->qtype_icon($this->questions[$counts->slot]->type);
+            $row[] = $PAGE->get_renderer('question', 'bank')->qtype_icon($this->questions[$counts->slot]->qtype);
 
             $row[] = format_string($counts->name);
 
@@ -397,12 +412,12 @@ class quiz_grading_report extends quiz_default_report {
      * @param int $pagesize number of questions to show per page.
      * @param int $page current page number.
      * @param bool $shownames whether student names should be shown.
-     * @param bool $showidnumbers wither student idnumbers should be shown.
+     * @param bool $showcustomfields whether custom field values should be shown.
      * @param string $order preferred order of attempts.
      * @param stdClass $counts object that stores the number of each type of attempt.
      */
     protected function display_grading_interface($slot, $questionid, $grade,
-            $pagesize, $page, $shownames, $showidnumbers, $order, $counts) {
+            $pagesize, $page, $shownames, $showcustomfields, $order, $counts) {
 
         if ($pagesize * $page >= $counts->$grade) {
             $page = 0;
@@ -419,7 +434,7 @@ class quiz_grading_report extends quiz_default_report {
         if (array_key_exists('includeauto', $this->viewoptions)) {
             $hidden['includeauto'] = $this->viewoptions['includeauto'];
         }
-        $mform = new quiz_grading_settings_form($hidden, $counts, $shownames, $showidnumbers);
+        $mform = new quiz_grading_settings_form($hidden, $counts, $shownames, $showcustomfields, $this->context);
 
         // Tell the form the current settings.
         $settings = new stdClass();
@@ -474,7 +489,7 @@ class quiz_grading_report extends quiz_default_report {
                     $slot,
                     $displayoptions,
                     $this->questions[$slot]->number,
-                    $this->get_question_heading($attempt, $shownames, $showidnumbers)
+                    $this->get_question_heading($attempt, $shownames, $showcustomfields)
             );
         }
 
@@ -537,15 +552,17 @@ class quiz_grading_report extends quiz_default_report {
 
     /**
      * Save all submitted marks to the database.
+     *
+     * @return bool returns true if some attempts or all are graded. False, if none of the attempts are graded.
      */
-    protected function process_submitted_data() {
+    protected function process_submitted_data(): bool {
         global $DB;
 
         $qubaids = optional_param('qubaids', null, PARAM_SEQUENCE);
         $assumedslotforevents = optional_param('slot', null, PARAM_INT);
 
         if (!$qubaids) {
-            return;
+            return false;
         }
 
         $qubaids = clean_param_array(explode(',', $qubaids), PARAM_INT);
@@ -553,11 +570,24 @@ class quiz_grading_report extends quiz_default_report {
         $events = [];
 
         $transaction = $DB->start_delegated_transaction();
+        $attemptsgraded = false;
         foreach ($qubaids as $qubaid) {
             $attempt = $attempts[$qubaid];
             $attemptobj = new quiz_attempt($attempt, $this->quiz, $this->cm, $this->course);
+
+            // State of the attempt before grades are changed.
+            $attemptoldtstate = $attemptobj->get_question_state($assumedslotforevents);
+
             $attemptobj->process_submitted_actions(time());
 
+            // Get attempt state after grades are changed.
+            $attemptnewtstate = $attemptobj->get_question_state($assumedslotforevents);
+
+            // Check if any attempts are graded.
+            if (!$attemptsgraded && $attemptoldtstate->is_graded() != $attemptnewtstate->is_graded()) {
+                $attemptsgraded = true;
+            }
+
             // Add the event we will trigger later.
             $params = [
                 'objectid' => $attemptobj->get_question_attempt($assumedslotforevents)->get_question_id(),
@@ -577,6 +607,8 @@ class quiz_grading_report extends quiz_default_report {
         foreach ($events as $event) {
             $event->trigger();
         }
+
+        return $attemptsgraded;
     }
 
     /**
@@ -617,7 +649,7 @@ class quiz_grading_report extends quiz_default_report {
     protected function get_usage_ids_where_question_in_state($summarystate, $slot,
             $questionid = null, $orderby = 'random', $page = 0, $pagesize = null) {
         $dm = new question_engine_data_mapper();
-
+        $extraselect = '';
         if ($pagesize && $orderby != 'random') {
             $limitfrom = $page * $pagesize;
         } else {
@@ -627,42 +659,46 @@ class quiz_grading_report extends quiz_default_report {
         $qubaids = $this->get_qubaids_condition();
 
         $params = [];
-        if ($orderby == 'date') {
+        $userfieldsapi = \core_user\fields::for_identity($this->context)->with_name();
+        $userfieldssql = $userfieldsapi->get_sql('u', true, '', 'userid', true);
+        $params = array_merge($params, $userfieldssql->params);
+        $customfields = [];
+        foreach ($userfieldsapi->get_required_fields([\core_user\fields::PURPOSE_IDENTITY]) as $field) {
+            $customfields[] = $field;
+        }
+        if ($orderby === 'date') {
             list($statetest, $params) = $dm->in_summary_state_test(
                     'manuallygraded', false, 'mangrstate');
-            $orderby = "(
+            $extraselect = "(
                     SELECT MAX(sortqas.timecreated)
                     FROM {question_attempt_steps} sortqas
                     WHERE sortqas.questionattemptid = qa.id
                         AND sortqas.state $statetest
-                    )";
-        } else if ($orderby == 'studentfirstname' || $orderby == 'studentlastname' || $orderby == 'idnumber') {
-            $qubaids->from .= " JOIN {user} u ON quiza.userid = u.id ";
+                    ) as tcreated";
+            $orderby = "tcreated";
+        } else if ($orderby === 'studentfirstname' || $orderby === 'studentlastname' || in_array($orderby, $customfields)) {
+            $qubaids->from .= " JOIN {user} u ON quiza.userid = u.id {$userfieldssql->joins}";
             // For name sorting, map orderby form value to
             // actual column names; 'idnumber' maps naturally.
-            switch ($orderby) {
-                case "studentlastname":
-                    $orderby = "u.lastname, u.firstname";
-                    break;
-                case "studentfirstname":
-                    $orderby = "u.firstname, u.lastname";
-                    break;
-                case "idnumber":
-                    $orderby = "u.idnumber";
-                    break;
+            if ($orderby === "studentlastname") {
+                $orderby = "u.lastname, u.firstname";
+            } else if ($orderby === "studentfirstname") {
+                $orderby = "u.firstname, u.lastname";
+            } else if (in_array($orderby, $customfields)) { // Sort order by current custom user field.
+                $orderby = $userfieldssql->mappings[$orderby];
             }
         }
 
         return $dm->load_questions_usages_where_question_in_state($qubaids, $summarystate,
-                $slot, $questionid, $orderby, $params, $limitfrom, $pagesize);
+                $slot, $questionid, $orderby, $params, $limitfrom, $pagesize, $extraselect);
     }
 
     /**
      * Initialise some parts of $PAGE and start output.
      *
-     * @param object $cm the course_module information.
-     * @param object $course the course settings.
-     * @param object $quiz the quiz settings.
+     * @param stdClass $cm the course_module information.
+     * @param stdClass $course the course settings.
+     * @param stdClass $quiz the quiz settings.
      * @param string $reportmode the report name.
      */
     public function print_header_and_tabs($cm, $course, $quiz, $reportmode = 'overview') {
@@ -674,26 +710,32 @@ class quiz_grading_report extends quiz_default_report {
     /**
      * Get question heading.
      *
-     * @param object $attempt an instance of quiz_attempt.
-     * @param bool $shownames True to show the question name.
-     * @param bool $showidnumbers True to show the question id number.
+     * @param stdClass $attempt An instance of quiz_attempt.
+     * @param bool $shownames True to show the student first/lastnames.
+     * @param bool $showcustomfields Whether custom field values should be shown.
      * @return string The string text for the question heading.
-     * @throws coding_exception
      */
-    protected function get_question_heading($attempt, $shownames, $showidnumbers) {
+    protected function get_question_heading(stdClass $attempt, bool $shownames, bool $showcustomfields): string {
+        global $DB;
         $a = new stdClass();
         $a->attempt = $attempt->attempt;
         $a->fullname = fullname($attempt);
-        $a->idnumber = s($attempt->idnumber);
 
-        $showidnumbers = $showidnumbers && !empty($attempt->idnumber);
+        $customfields = [];
+        foreach ($this->extrauserfields as $field) {
+            if (strval($attempt->{$field}) !== '') {
+                $customfields[] = s($attempt->{$field});
+            }
+        }
 
-        if ($shownames && $showidnumbers) {
-            return get_string('gradingattemptwithidnumber', 'quiz_grading', $a);
+        $a->customfields = implode(', ', $customfields);
+
+        if ($shownames && $showcustomfields) {
+            return get_string('gradingattemptwithcustomfields', 'quiz_grading', $a);
         } else if ($shownames) {
             return get_string('gradingattempt', 'quiz_grading', $a);
-        } else if ($showidnumbers) {
-            $a->fullname = $attempt->idnumber;
+        } else if ($showcustomfields) {
+            $a->fullname = $a->customfields;
             return get_string('gradingattempt', 'quiz_grading', $a);
         } else {
             return '';
